from tkadwold.windows.canvas.drawengine import AdwDrawEngine
from tkadwold.layout import AdwLayout


class AdwWidget(AdwDrawEngine, AdwLayout):

    """
    基础绘制组件类

    特性：自动将背景颜色设为父组件背景颜色
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.bind("<Configure>", self._draw, add="+")
        self._other()

    def _other(self):
        from tkadwold.windows.canvas.frame import AdwDrawBasicFrame
        if hasattr(self.master, "frame_back"):
            self.configure(background=self.master.frame_back)
        self.configure(background=self.master.cget("bg"), borderwidth=0)

    def update(self) -> None:
        super().update()
        self._draw(None)
        self._other()

    def _draw(self, evt=None):
        pass


from tkadwold.windows.theme import AdwTButton, AdwTFrame, AdwTLabel, Adwite
from sys import platform


class AdwCustomTk(Adwite):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.frameless()
        self.build()
        
        self.x, self.y = 0, 0

    def frameless(self):
        self.wm_overrideredirect(True)

        try:
            from ctypes import windll
            GWL_EXSTYLE = -20
            WS_EX_APPWINDOW = 0x00040000
            WS_EX_TOOLWINDOW = 0x00000080
            hwnd = windll.user32.GetParent(self.winfo_id())
            style = windll.user32.GetWindowLongW(hwnd, GWL_EXSTYLE)
            style = style & ~WS_EX_TOOLWINDOW
            style = style | WS_EX_APPWINDOW
            res = windll.user32.SetWindowLongW(hwnd, GWL_EXSTYLE, style)
        except:
            self.wm_attributes("-topmost", True)

        self.minsize(150, 180)

    def build(self):
        self.frame_border = AdwTFrame(self)
        self.frame_border.pack(fill="both", expand=True, padx=3, pady=3)

        self.frame_title = AdwTFrame(self.frame_border.frame, borderwidth=0, height=50)

        self.frame_title.pack(fill="x", side="top", padx=2, pady=2)
        self.bind_move(self.frame_title.frame)

        self.label_title = AdwTLabel(self.frame_title.frame, text=self.wm_title())
        self.label_title.pack(side="left", anchor="w", padx=10, pady=5)
        self.bind_move(self.label_title)

        self.maximized = False

        self.button_close = AdwTButton(self.frame_title.frame, text="✕", width=30, height=30, command=lambda: self.destroy())
        self.button_close.pack(side="right", anchor="e", padx=5, pady=5)

        self.button_minimize = AdwTButton(self.frame_title.frame, text="–", width=30, height=30, command=lambda: self.minimize())
        self.button_minimize.pack(side="right", anchor="e", padx=5, pady=5)

    def minimize(self):
        try:
            from ctypes import windll
            hwnd = windll.user32.GetParent(self.winfo_id())
            windll.user32.ShowWindow(hwnd, 2)
        except:
            pass

    def bind_move(self, widget):
        widget.bind("<Button-1>", self._click)
        widget.bind("<B1-Motion>", self._move)

    def create_sizegrip(self):
        from tkinter import ttk
        ttk.Style().configure("CTkCustom.TSizegrip", background="#ffffff")
        self.sizegrip = ttk.Sizegrip(self.frame_border, style="CTkCustom.TSizegrip")
        self.sizegrip.pack(side="bottom", anchor="se", padx=5, pady=5, ipady=2)
        return self.sizegrip

    @property
    def titlebar(self):
        return self.frame_title

    @property
    def titlebar_title(self):
        return self.label_title

    @property
    def titlebar_closebutton(self):
        return self.button_close

    @property
    def mainframe(self):
        return self.frame_border

    def _click(self, event):
        self.x, self.y = event.x, event.y

    def _move(self, event):
        new_x = (event.x - self.x) + self.winfo_x()
        new_y = (event.y - self.y) + self.winfo_y()
        if new_y <= 0:
            new_y = 0
        s = f"+{new_x}+{new_y}"
        self.geometry(s)


class AdwCustomSVTTk(AdwCustomTk):
    def build(self):
        from sv_ttk import use_dark_theme
        use_dark_theme()
        self.frame_border = ttk.Frame(self, style="Card.TFrame")
        self.frame_border.pack(fill="both", expand=True, padx=3, pady=3)

        self.frame_title = ttk.Frame(self.frame_border, borderwidth=0)
        self.frame_title.pack(fill="x", side="top", padx=2, pady=2)
        self.bind_move(self.frame_title)

        self.label_title = ttk.Label(self.frame_title, text=self.wm_title())
        self.label_title.pack(side="left", anchor="w", padx=10, pady=5)
        self.bind_move(self.label_title)

        self.maximized = False

        self.button_close = ttk.Button(self.frame_title, text="✕", command=lambda: self.destroy())
        self.button_close.pack(side="right", anchor="e", padx=5, pady=5)

        self.button_minimize = ttk.Button(self.frame_title, text="–", command=lambda: self.minimize())
        self.button_minimize.pack(side="right", anchor="e", padx=5, pady=5)

    def create_sizegrip(self):
        self.sizegrip = ttk.Sizegrip(self.frame_border)
        self.sizegrip.pack(side="bottom", anchor="se", padx=5, pady=5, ipady=2)
        return self.sizegrip


if __name__ == '__main__':
    root = AdwCustomTk()
    root.create_sizegrip()
    root.mainloop()


from tkinter.font import Font, nametofont
from tkadwold.windows.canvas.widget import AdwWidget


# Button
class AdwDrawBasicButton(AdwWidget):

    """
    自绘基础按钮
    """

    def __init__(self, *args, width=120, height=40, text: str = "", command=None, **kwargs):
        super().__init__(*args, width=width, height=height, bd=0, highlightthickness=0, **kwargs)

        self.default_palette()

        self.text = text

        self._button_back = self.button_back
        self._button_border = self.button_border
        self._button_border_width = self.button_border_width
        self._button_text_back = self.button_text_back

        self.button_text_font = nametofont("TkDefaultFont")

        # 绑定事件
        self.bind("<Button>", self._click, add="+")
        self.bind("<ButtonRelease>", self._unclick, add="+")
        self.bind("<Enter>", self._hover, add="+")
        self.bind("<Leave>", self._hover_release, add="+")

        # 绑定点击事件（不是按下）
        if command is not None:
            self.bind("<<Click>>", lambda event: command())

        self._draw(None)

    def configure(self, **kwargs):
        if "command" in kwargs:
            self.command = kwargs.pop("command")
            self.bind("<<Click>>", lambda event: self.command())
        elif "text" in kwargs:
            self.text = kwargs.pop("text")
            self._draw(None)
        else:
            super().configure(**kwargs)

    def update(self) -> None:
        """
        多执行一道绘制
        """
        super().update()
        self._draw(None)

    def _draw(self, evt=None):
        self.delete("all")

        # 绘制按钮边框
        self.button_frame = self.create_rectangle(
            self._button_border_width - 1, self._button_border_width - 1,
            self.winfo_width() - self._button_border_width + 1,
            self.winfo_height() - self._button_border_width + 1,
            width=self._button_border_width,
            outline=self._button_border, fill=self._button_back,
        )

        # 绘制按钮文本
        self.button_text = self.create_text(
            self.winfo_width() / 2, self.winfo_height() / 2,
            text=self.text, fill=self._button_text_back,
            font=self.button_text_font
        )

    def _click(self, evt=None):
        self.hover = True
        self._button_back = self.button_pressed_back
        self._button_border = self.button_pressed_border
        self._button_border_width = self.button_pressed_border_width
        self._button_text_back = self.button_pressed_text_back

        self.focus_set()

        self._draw(None)

    def _unclick(self, evt=None):
        if self.hover:
            self._button_back = self.button_active_back
            self._button_border = self.button_active_border
            self._button_border_width = self.button_active_border_width
            self._button_text_back = self.button_active_text_back

            self._draw(None)

            self.event_generate("<<Click>>")

    def _hover(self, evt=None):
        self.hover = True
        self._button_back = self.button_active_back
        self._button_border = self.button_active_border
        self._button_border_width = self.button_active_border_width
        self._button_text_back = self.button_active_text_back

        self._draw(None)

    def _hover_release(self, evt=None):
        self.hover = False
        self._button_back = self.button_back
        self._button_border = self.button_border
        self._button_border_width = self.button_border_width
        self._button_text_back = self.button_text_back

        self._draw(None)

    def font(self, font: Font = None):
        if font is None:
            return self.button_text_font
        else:
            self.button_text_font = font

    def default_palette(self):
        self.palette_light()

    def palette_light(self):
        self.palette(
            {
                "button": {
                    "back": "#fdfdfd",
                    "border": "#eaeaea",
                    "text_back": "#1a1a1a",
                    "border_width": 1,

                    "active": {
                        "back": "#f9f9f9",
                        "border": "#aaaaaa",
                        "text_back": "#5f5f5f",
                        "border_width": 1,
                    },

                    "pressed": {
                        "back": "#f9f9f9",
                        "border": "#e2e2e2",
                        "text_back": "#8a8a8a",
                        "border_width": 1,
                    },
                }
            }
        )

    def palette_dark(self):
        self.palette(
            {
                "button": {
                    "back": "#353535",
                    "border": "#454545",
                    "text_back": "#ffffff",
                    "border_width": 1,

                    "active": {
                        "back": "#3a3a3a",
                        "border": "#454545",
                        "text_back": "#cecece",
                        "border_width": 1,
                    },

                    "pressed": {
                        "back": "#2f2f2f",
                        "border": "#454545",
                        "text_back": "#9a9a9a",
                        "border_width": 1,
                    },
                }
            }
        )

    def palette(self, dict=None):
        if dict is not None:
            if "button" in dict:
                self.button_back = dict["button"]["back"]
                self.button_border = dict["button"]["border"]
                self.button_text_back = dict["button"]["text_back"]
                self.button_border_width = dict["button"]["border_width"]

                if "active" in dict["button"]:
                    self.button_active_back = dict["button"]["active"]["back"]
                    self.button_active_border = dict["button"]["active"]["border"]
                    self.button_active_text_back = dict["button"]["active"]["text_back"]
                    self.button_active_border_width = dict["button"]["active"]["border_width"]

                if "pressed" in dict["button"]:
                    self.button_pressed_back = dict["button"]["pressed"]["back"]
                    self.button_pressed_border = dict["button"]["pressed"]["border"]
                    self.button_pressed_text_back = dict["button"]["pressed"]["text_back"]
                    self.button_pressed_border_width = dict["button"]["pressed"]["border_width"]

            self._palette = dict

            self._button_back = self.button_back
            self._button_border = self.button_border
            self._button_border_width = self.button_border_width
            self._button_text_back = self.button_text_back

            try:
                self._draw(None)
            except AttributeError:
                pass
        else:
            return self._palette


class AdwDrawButton(AdwDrawBasicButton):
    def default_palette(self):
        self.palette_light()


class AdwDrawDarkButton(AdwDrawBasicButton):
    def default_palette(self):
        self.palette_dark()


class AdwDrawAccentButton(AdwDrawBasicButton):
    def default_palette(self):
        self.palette(
            {
                "button": {
                    "back": "#0067c0",
                    "border": "#1473c5",
                    "text_back": "#ffffff",
                    "border_width": 1,

                    "active": {
                        "back": "#1975c5",
                        "border": "#1473c5",
                        "text_back": "#ffffff",
                        "border_width": 1,
                    },

                    "pressed": {
                        "back": "#3284cb",
                        "border": "#3284cb",
                        "text_back": "#fdfdfd",
                        "border_width": 1,
                    },
                }
            }
        )


# Round Button
class AdwDrawBasicRoundButton(AdwDrawBasicButton):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def _other(self):
        self.configure(background=self.master.cget("bg"), borderwidth=0)

    def _draw(self, evt):
        self.delete("all")

        # 绘制按钮边框
        self.button_frame = self.create_round_rect2(
            self._button_border_width - 1,
            self._button_border_width - 1,
            self.winfo_width() - self._button_border_width * 2 + 1,
            self.winfo_height() - self._button_border_width * 2 + 1, self.button_radius,
            width=self._button_border_width,
            outline=self._button_border, fill=self._button_back,
        )

        # 绘制按钮文本
        self.button_text = self.create_text(
            self.winfo_width() / 2, self.winfo_height() / 2,
            text=self.text, fill=self._button_text_back,
            font=self.button_text_font
        )

    def default_palette(self):
        self.palette_light()

    def palette_light(self):
        self.palette(
            {
                "button": {
                    "radius": 8,
                    "back": "#fdfdfd",
                    "border": "#eaeaea",
                    "text_back": "#1a1a1a",
                    "border_width": 1,

                    "active": {
                        "back": "#f9f9f9",
                        "border": "#454545",
                        "text_back": "#5f5f5f",
                        "border_width": 1,
                    },

                    "pressed": {
                        "back": "#f9f9f9",
                        "border": "#e2e2e2",
                        "text_back": "#8a8a8a",
                        "border_width": 1,
                    },
                }
            }
        )

    def palette_dark(self):
        self.palette(
            {
                "button": {
                    "radius": 8,
                    "back": "#353535",
                    "border": "#454545",
                    "text_back": "#ffffff",
                    "border_width": 1,

                    "active": {
                        "back": "#3a3a3a",
                        "border": "#454545",
                        "text_back": "#cecece",
                        "border_width": 1,
                    },

                    "pressed": {
                        "back": "#2f2f2f",
                        "border": "#454545",
                        "text_back": "#9a9a9a",
                        "border_width": 1,
                    },
                }
            }
        )

    def palette(self, dict=None):
        super().palette(dict)
        if "button" in dict:
            self.button_radius = dict["button"]["radius"]


class AdwDrawRoundButton(AdwDrawBasicRoundButton):
    def default_palette(self):
        self.palette_light()


class AdwDrawRoundAccentButton(AdwDrawBasicRoundButton):
    def default_palette(self):
        self.palette(
            {
                "button": {
                    "radius": 8,
                    "back": "#0067c0",
                    "border": "#1473c5",
                    "text_back": "#ffffff",
                    "border_width": 1,

                    "active": {
                        "back": "#1975c5",
                        "border": "#1473c5",
                        "text_back": "#ffffff",
                        "border_width": 1,
                    },

                    "pressed": {
                        "back": "#3284cb",
                        "border": "#3284cb",
                        "text_back": "#fdfdfd",
                        "border_width": 1,
                    },
                }
            }
        )


class AdwDrawRoundDarkButton(AdwDrawBasicRoundButton):
    def default_palette(self):
        self.palette_dark()


class AdwDrawRoundButton2(AdwDrawBasicRoundButton):
    # 使用此控件，不建议使用边框属性
    def _draw(self, evt):
        self.delete("all")

        # 绘制按钮文本
        self.create_round_rect3(
            "button_frame",
            self._button_border_width - 1,
            self._button_border_width - 1,
            self.winfo_width() - self._button_border_width * 2,
            self.winfo_height() - self._button_border_width * 2,
            self.button_radius,
            outline=self._button_border, fill=self._button_back,
        )

        self.button_frame = "button_frame"

        # 绘制按钮文本
        self.button_text = self.create_text(
            self.winfo_width() / 2, self.winfo_height() / 2,
            text=self.text, fill=self._button_text_back,
            font=self.button_text_font
        )

    def default_palette(self):
        self.palette_light()


class AdwDrawRoundAccentButton2(AdwDrawRoundButton2):
    def default_palette(self):
        self.palette(
            {
                "button": {
                    "radius": 8,
                    "back": "#0067c0",
                    "border": "#1473c5",
                    "text_back": "#ffffff",
                    "border_width": 1,

                    "active": {
                        "back": "#1975c5",
                        "border": "#1473c5",
                        "text_back": "#ffffff",
                        "border_width": 1,
                    },

                    "pressed": {
                        "back": "#3284cb",
                        "border": "#3284cb",
                        "text_back": "#fdfdfd",
                        "border_width": 1,
                    },
                }
            }
        )


class AdwDrawRoundDarkButton2(AdwDrawRoundButton2):
    def default_palette(self):
        self.palette_dark()


class AdwDrawRoundButton3(AdwDrawBasicRoundButton):
    def _draw(self, evt):
        self.delete("all")

        # 绘制按钮边框
        self.create_round_rect4(
            self._button_border_width - 1, self._button_border_width - 1,
            self.winfo_width() - self._button_border_width,
            self.winfo_height() - self._button_border_width,
            self.button_radius,
            width=self._button_border_width,
            outline=self._button_border, fill=self._button_back,
        )

        self.button_frame = "button_frame"

        # 绘制按钮文本
        self.button_text = self.create_text(
            self.winfo_width() / 2, self.winfo_height() / 2,
            text=self.text, fill=self._button_text_back,
            font=self.button_text_font
        )

    def default_palette(self):
        self.palette_light()


class AdwDrawRoundAccentButton3(AdwDrawRoundButton3):
    def default_palette(self):
        self.palette(
            {
                "button": {
                    "radius": 8,
                    "back": "#0067c0",
                    "border": "#1473c5",
                    "text_back": "#ffffff",
                    "border_width": 1,

                    "active": {
                        "back": "#1975c5",
                        "border": "#1473c5",
                        "text_back": "#ffffff",
                        "border_width": 1,
                    },

                    "pressed": {
                        "back": "#3284cb",
                        "border": "#3284cb",
                        "text_back": "#fdfdfd",
                        "border_width": 1,
                    },
                }
            }
        )


class AdwDrawRoundDarkButton3(AdwDrawRoundButton3):
    def default_palette(self):
        self.palette_dark()


# Circular Button
class AdwDrawBasicCircularButton(AdwDrawBasicButton):
    def __init__(self, *args, width=120, height=120, **kwargs):
        super().__init__(*args, width=width, height=height, **kwargs)

    def _draw(self, evt):
        self.delete("all")

        # 绘制按钮框架
        self.button_frame = self.create_oval(
            1.5, 1.5, self.winfo_width() - 3, self.winfo_height() - 3,
            width=self.button_border_width,
            outline=self._button_border, fill=self._button_back,
        )

        # 绘制按钮文本
        self.button_text = self.create_text(
            self.winfo_width() / 2, self.winfo_height() / 2,
            text=self.text, fill=self._button_text_back,
            font=self.button_text_font
        )

        def palette(self, dict=None):
            if dict is not None:
                if "circular_button" in dict:
                    self.button_back = dict["circular_button"]["back"]
                    self.button_border = dict["circular_button"]["border"]
                    self.button_text_back = dict["circular_button"]["text_back"]
                    self.button_border_width = dict["circular_button"]["border_width"]

                    if "active" in dict["circular_button"]:
                        self.button_active_back = dict["circular_button"]["active"]["back"]
                        self.button_active_border = dict["circular_button"]["active"]["border"]
                        self.button_active_text_back = dict["circular_button"]["active"]["text_back"]
                        self.button_active_border_width = dict["circular_button"]["active"]["border_width"]

                    if "pressed" in dict["circular_button"]:
                        self.button_pressed_back = dict["circular_button"]["pressed"]["back"]
                        self.button_pressed_border = dict["circular_button"]["pressed"]["border"]
                        self.button_pressed_text_back = dict["circular_button"]["pressed"]["text_back"]
                        self.button_pressed_border_width = dict["circular_button"]["pressed"]["border_width"]

                self._palette = dict

                self._button_back = self.button_back
                self._button_border = self.button_border
                self._button_border_width = self.button_border_width
                self._button_text_back = self.button_text_back

                try:
                    self._draw(None)
                except AttributeError:
                    pass
            else:
                return self._palette


class AdwDrawCircularButton(AdwDrawBasicCircularButton):
    def default_palette(self):
        self.palette_light()


class AdwDrawCircularDarkButton(AdwDrawBasicCircularButton):
    def default_palette(self):
        self.palette_dark()


if __name__ == '__main__':
    from tkinter import Tk

    root = Tk()

    button = AdwDrawButton(text="Hello")
    button.bind("<<Click>>", lambda evt: print("button clicked"))
    button.pack(fill="x", padx=5, pady=5)

    button4 = AdwDrawRoundButton(text="Hello")
    button4.bind("<<Click>>", lambda evt: print("button4 clicked"))
    button4.pack(fill="x", padx=5, pady=5)

    button7 = AdwDrawRoundButton2(text="Hello")
    button7.bind("<<Click>>", lambda evt: print("button4 clicked"))
    button7.pack(fill="x", padx=5, pady=5)

    button10 = AdwDrawRoundButton3(text="Hello")
    button10.bind("<<Click>>", lambda evt: print("button4 clicked"))
    button10.pack(fill="x", padx=5, pady=5)

    button13 = AdwDrawCircularButton(text="Hello")
    button13.bind("<<Click>>", lambda evt: print("button6 clicked"))
    button13.pack(fill="x", padx=5, pady=5)

    root.mainloop()



from tkinter import Frame, Widget


class AdwStack(Frame):
    def __init__(self, *args, **kwargs):
        """
        用于切换不同的界面
        """
        super().__init__(*args, **kwargs)
        self._pages = {}

    def add_page(self, page: Widget, id: int = 0):
        """
        添加页面

        :param page: 页面组件
        :param id: 组件ID
        """
        self._pages[id] = page

    def show_page(self, id: int):
        """
        显示页面，会将其他页面隐藏

        :param id: 被显示的页面ID
        """
        self._pages[id].pack(fill=tk.BOTH, expand=tk.YES)
        for item in self._pages.keys():
            if not item == id:
                self.hide_page(item)

    def hide_page(self, id: int):
        """
        内置函数，最好不要使用，因为几乎没有什么用
        """
        self._pages[id].pack_forget()

    def get_page(self, id: int):
        """
        获取页面

        :param id: 所要获取的页面ID
        """
        return self._pages[id]

    def get_pages(self):
        """
        获取所有页面
        """
        return self._pages


# 0.2.3加入
from tkadwold.windows.fluent.entry import FluentEntry, FluentDarkEntry
from tkadwold.windows.fluent.button import FluentButton, FluentDarkButton
from tkadwold.windows.fluent.frame import FluentFrame, FluentDarkFrame


from tkinter import Toplevel
from tkadwold.windows.widgets.adw import Adw


class AdwToplevel(Toplevel, Adw):
    def __init__(self, *args, master=None, title: str = "adw", config: bool = False,
                 dark: bool = True, dark_with_refresh: bool = False,
                 wincaption=None, **kwargs):
        super().__init__(*args, **kwargs)




from tkinterflow.tkinterflow import _flow, _flow_destroy


class Flow(object):

    mode = "place"

    def flow(self, mode="grid", *args, **kwargs):
        if mode == "place":
            self.mode = "place"
        else:
            self.mode = "grid"
        _flow(self, self.mode, *args, **kwargs)

    def flow_destroy(self):
        _flow_destroy(self, self.mode)



from tkinter import Widget
from _tkinter import TclError


def put_configure(widget: Widget, x=0, y=0, width=0, height=0, top=None, bottom=None, left=None, right=None, *args, **kwargs):
    def configure(evt):
        if top is not None:
            y = top
        if left is not None:
            x = left
        if right is not None:
            width = widget.master.winfo_width() - x - right
        if bottom is not None:
            height = widget.master.winfo_height() -y -bottom
        widget.place(x=x, y=y, width=width, height=height, *args, **kwargs)
    configure(None)
    return widget.master.bind("<Configure>", configure, add="+"), widget.bind("<Configure>", configure, add="+"), \
        {"in": widget.place_info()["in"], "x": x, "y": y, "width": width, "height": height, "top": top, "bottom": bottom, "left": left, "right": right}


def put_forget(widget: Widget, putid):
    """
最好请使用Adw.run()
    """
    widget.place_forget()
    widget.unbind("<Configure>", putid)


class AdwLayoutPut:
    def put_configure(self, *args, **kwargs):
        r = put_configure(self, *args, **kwargs)
        self.putid = r[0]
        self.putid2 = r[1]
        self.putinfo = r[2]

    put = put_configure

    def put_forget(self):
        return put_forget(self, self.putid), put_forget(self, self.putid2)

    def put_info(self):
        return self.putinfo


if __name__ == "__main__":
    from tkadwold import Adwite, AdwTButton

    class TestButton(AdwTButton):
        def __init__(self, *args, **kwargs):
            super().__init__(*args, command=self.put_forget, **kwargs)

    root = Adwite()
    btn1 = TestButton(root, text="1", width=40, height=40)
    btn1.put(left=5, right=45, top=5, bottom=45)
    btn2 = TestButton(root, text="2", width=40, height=40)
    btn2.put(left=45, right=5, top=45, bottom=5)
    print(btn1.place_info())
    print(btn1.put_info())
    root.run()


from tkadwold.windows.canvas.frame import AdwDrawFrame, AdwDrawRoundFrame, AdwDrawDarkRoundFrame3


class AdwBasicTask(AdwDrawFrame):
    def _draw(self, evt=None):
        super()._draw(evt)

        self.task_top = self.create_rectangle(self.frame_border_width,
                                              self.frame_border_width,
                                              self.frame_border_width,
                                              self.frame_border_width,
                                              fill=self.task_top, outline=self.task_top,
                                              width=self.task_top_width)

    def palette(self, dict=None):
        if dict is not None:
            if "task" in dict:
                self.frame_back = dict["task"]["back"]
                self.frame_border = dict["task"]["border"]
                self.frame_border_width = dict["task"]["border_width"]
                self.frame_padding = dict["task"]["padding"]
                self.task_top = dict["task"]["top_line"]
                self.task_top_width = dict["task"]["top_width"]

            self._palette = dict

            try:
                self._draw(None)
            except AttributeError:
                pass
        else:
            return self._palette


if __name__ == '__main__':
    from tkadwold.windows.widgets.adw import Adw
    root = Adw()
    task = AdwBasicTask()
    task.pack(fill="both", side="top", padx=5, pady=5)
    root.mainloop()


from tkadwold.windows.canvas.widget import AdwWidget


# Frame
class AdwDrawBasicScrollBar(AdwWidget):
    def __init__(self, *args, width=200, height=200, **kwargs):
        super().__init__(*args, width=width, height=height, bd=0, highlightthickness=0, **kwargs)

        self.default_palette()

        self._draw(None)

    def _draw(self, evt):
        self.delete("all")

        # 绘制框架边框
        self.frame_frame = self.create_rectangle(
            0, 0, self.winfo_width(), self.winfo_height(),
            width=self.frame_border_width,
            outline=self.frame_border, fill=self.frame_back,
        )

        self.frame.configure(background=self.frame_back, bd=0, )


    def default_palette(self):
        self.palette_light()

    def palette_light(self):
        self.palette(
            {
                "frame": {
                    "back": "#ffffff",
                    "border": "#eaeaea",
                    "border_width": 2,
                    "padding": 0
                }
            }
        )

    def palette_dark(self):
        self.palette(
            {
                "frame": {
                    "back": "#0f0f0f",
                    "border": "#333333",
                    "border_width": 2,
                    "padding": 0
                }
            }
        )

    def palette(self, dict=None):
        if dict is not None:
            if "frame" in dict:
                self.frame_back = dict["frame"]["back"]
                self.frame_border = dict["frame"]["border"]
                self.frame_border_width = dict["frame"]["border_width"]
                self.frame_padding = dict["frame"]["padding"]

            self._palette = dict

            try:
                self._draw(None)
            except AttributeError:
                pass
        else:
            return self._palette


from tkadwold.windows.widgets import *
from tkadwold.utility.appconfig import appconfig as AdwReadAppConfig

from tkadwold.windows import *

# 0.3.0加入
from tkadwold.windows.theme import *

from tkadwold.game import *

# 0.3.5加入
from tkadwold.layout import *

# 0.3.9加入
from tkadwold.designer.designerframe import AdwDesignerFrame
from tkadwold.designer.designer import AdwDesigner
from tkadwold.designer.builder import AdwBuilder

try:
    from tkadw_material import *
except ModuleNotFoundError:
    pass


# from tkadwold.tkite import * 已废弃移除
# from tkadwold.win11 import * 已废弃移除
# from tkadwold.advanced import * 已废弃移除，改为from tkadwold.adw import Adw导入
# from tkadwold.bilibili import BiliBiliButton, BiliBiliDarkButton, BiliBiliFrame, BiliBiliDarkFrame, \
#     BiliBiliEntry, BiliBiliDarkEntry, BiliBiliDarkTextBox, BiliBiliTextBox 已废弃移除

# 0.3.7补充
from tkadwold.utility import *


def get_version():
    return "0.4.6"


def get_major_version():
    return "0"

def get_micro_version():
    return "4"


if __name__ == '__main__':
    from tkinter import Tk, Toplevel

    root = Tk()

    adw_run(root)


from tkadw import *

root = AdwTMainWindow(
    styles=["optimised"]
)
root.theme("win11", "dark")
#root.styles(["aero"])
root.dark(True)

button = AdwTButton(text="Button", command=lambda: print("Button"))
button.pack(fill="both", expand="yes", padx=15, pady=(7.5, 15))

entry = AdwTEntry(text="Entry")
entry.pack(fill="both", expand="yes", padx=15, pady=(7.5, 15))

root.run()


from tkinter import Label
from tkadwold.layout import AdwLayout


class AdwBasicLabel(Label, AdwLayout):
    def __init__(self, *args, text: str = "", **kwargs):
        super().__init__(*args, text=text, **kwargs)

        self._other()

        self.default_palette()

        self.bind("<Configure>", self._draw, add="+")
        self.bind("<Enter>", self._draw(), add="+")
        self.bind("<Leave>", self._draw(), add="+")

    def _draw(self, evt=None):
        if self.label_back == "transparent":
            self.configure(background=self.master.cget("bg"))
        self.configure(foreground=self.label_text_back)

    def _other(self):
        pass

    def default_palette(self):
        self.palette_light()

    def palette_light(self):
        self.palette(
            {
                "label": {
                    "back": "transparent",
                    "text_back": "#000000",
                }
            }
        )

    def palette_dark(self):
        self.palette(
            {
                "label": {
                    "back": "transparent",
                    "text_back": "#ffffff",
                }
            }
        )

    def palette(self, dict=None):
        if dict is not None:
            if "label" in dict:
                self.label_back = dict["label"]["back"]
                self.label_text_back = dict["label"]["text_back"]

            self._palette = dict

            try:
                self._draw(None)
            except AttributeError:
                pass
        else:
            return self._palette


class AdwLabel(AdwBasicLabel):
    def default_palette(self):
        self.palette_light()


class AdwDarkLabel(AdwBasicLabel):
    def default_palette(self):
        self.palette_dark()


if __name__ == '__main__':
    from tkinter import Tk
    root = Tk()
    label = AdwLabel(text="hello")
    label.pack()
    label2 = AdwDarkLabel(text="hello")
    label2.pack()
    root.mainloop()



if __name__ == '__main__':
    from tkadwold import *
    root = Adwite(dark=False)
    root.geometry("600x300")
    root.set_default_theme("win11", "system")
    #root.style_dark_with_icon(True)
    #root.style("aero")
    root.style_header("#202020")


    def command():
        widgets_panel.show_page(choose_panel.select())


    def switch_theme(event):
        print(event)
        root.set_default_theme("win11", root.get_system_theme())


    root.bind("<<SystemThemeSwitch>>", switch_theme)

    choose_panel = AdwTListBox(
        items=sorted(
            ["AdwTButton", "AdwTCircularButton", "AdwTDesignerFrame", "AdwTEntry", "AdwTMDI", "AdwTTabs"]
        ),
        item_selected=command
    )
    choose_panel.column(expand=False)

    widgets_panel = AdwTStack()
    widgets_panel.column()

    # AdwTButton
    button_panel = AdwTFrame(widgets_panel.frame)
    button = AdwTButton(button_panel.frame, text="AdwTButton")
    button.row(expand="no")
    widgets_panel.add_page(button_panel, "AdwTButton")

    # AdwTCircularButton
    cbutton_panel = AdwTFrame(widgets_panel.frame)
    cbutton = AdwTCircularButton(cbutton_panel.frame, text="AdwTCircularButton")
    cbutton.row(expand="no", fill="none")
    widgets_panel.add_page(cbutton_panel, "AdwTCircularButton")

    # AdwTDesignerFrame
    designerframe_panel = AdwTFrame(widgets_panel.frame)
    designerframe = AdwTDesignerFrame(designerframe_panel.frame, widget=AdwTButton, text="AdwTDesignerFrame")
    designerframe.place(x=5, y=5, width=160, height=60)
    widgets_panel.add_page(designerframe_panel, "AdwTDesignerFrame")

    # AdwTEntry
    entry_panel = AdwTFrame(widgets_panel.frame)
    entry = AdwTEntry(entry_panel.frame)
    entry.row(expand="no")
    widgets_panel.add_page(entry_panel, "AdwTEntry")

    # AdwTMDI
    mdi_panel = AdwTFrame(widgets_panel.frame)
    mdi = AdwTMDI(mdi_panel.frame)
    mdi_child1 = mdi.create_child()
    mdi_child2 = mdi.create_designer_child()
    mdi.row()
    widgets_panel.add_page(mdi_panel, "AdwTMDI")

    # AdwTTabs
    tabs = AdwTTabs(widgets_panel.frame)
    for i in range(5):
        f = AdwTFrame(tabs.tab_pages.frame)
        l = AdwTLabel(f.frame, text=i)
        l.row()
        tabs.add(f, f"AdwTTab {i}")
    tabs.row()
    widgets_panel.add_page(tabs, "AdwTTabs")

    choose_panel.select("AdwTButton")
    widgets_panel.show_page("AdwTButton")

    #win = Adwitew()

    root.mainloop()



from tkinter import Tk
from .base import AdwBase
from .run import AdwRun


class AdwMainWindow(AdwBase, Tk):
    def __init__(self, *args, styles=None, title: str = "adwite", **kwargs):
        super().__init__(*args, **kwargs)

        self.protocol("WM_DELETE_WINDOW", self.quit)

        self.is_quit = None

        self.title(title)

        self.default_palette()

        from .style import WindowStyle

        self.windowstyle = WindowStyle(self)
        if styles:
            self.styles(styles)

        self.icon()

    def default_palette(self):
        pass

    def icon(self, dark=False):
        from .icon import PHOTOLIGHT, PHOTODARK
        if dark:
            self.iconphoto(False, PHOTODARK())
        else:
            self.iconphoto(False, PHOTOLIGHT())

    def palette(self, palette: dict):
        if "window" in palette:
            if "back" in palette["window"]:
                self.configure(background=palette["window"]["back"])

    def quit(self):
        self.is_quit = True
        self.destroy()

    def run(self):
        self.is_quit = False
        while not self.is_quit:
            self.update()

    def styles(self, names: list):
        """
        设置多个窗口样式

        Args:
            names (list): 样式名称
        """
        for style in names:
            self.windowstyle.style(style)



ARC = "arc"
CENTRAL = "central"
VERTICAL = "vertical"
HORIZONTAL = "horizontal"
CIRCULAR = "circular"


class Gradient:
    def __init__(self):
        from tkinter import _default_root, Tk
        self.tk: Tk = _default_root
        self.init()

    def demo(self):
        """
        示例
        """
        self.tk.call("gradient", "demo")

    from tkinter import Canvas

    def draw(self,
             canvas: Canvas, tags: str,
             x: int, y: int, width: int, height: int,
             orient, color1, color2, **kw):
        """
        绘制渐变图形

        Args:
            canvas (Canvas): 被绘制的画布
            tags (str): 渐变图形标签
            x (int): 渐变图形在画布上X轴坐标
            y (int): 渐变图形在画布上Y轴坐标
            width (int): 渐变图形的宽度
            height (int): 渐变图形的高度
            orient (str): 渐变图形的形状，有"arc" "central" "vertical" "horizontal" "circular"
            color1 (str): 渐变颜色1
            color2 (str): 渐变颜色2
        """
        kwargs = []
        for k in kw:
            kwargs.append("-" + k)
            kwargs.append(kw[k])
        return self.tk.call(
            "gradient", "draw", canvas._w,
            tags, x, y, width, height, orient, color1, color2, *kwargs
        )

    def init(self):
        """
        初始化渐变模块
        参见：https://wiki.tcl-lang.org/page/Drawing+Gradients+on+a+Canvas
        """
        self.tk.eval(
            """
            # Create color gradients of various shapes and sizes on a canvas.
            #   Usage:
            #       gradient draw    <canvas> <tags> <x> <y> <width> <height> <vertical|horizontal|circular|central|arc> <color1> <color2>  ?-animate? ?option? ...
            #       gradient redraw  <canvas> <tags> <x> <y> <width> <height> <vertical|horizontal|circular|central|arc> <color1> <color2>  ?-animate? ?option? ...
            #       gradient resize  <canvas> <tags> <x> <y> <width> <height>
            #       gradient recolor <canvas> <tags> <color1> <color2>
    
            namespace eval gradient {
                namespace ensemble create -subcommands {
                    draw
                    redraw
                    recolor
                    resize
                    demo
                }
    
                # Create a gradient on a canvas. The gradient will be placed at the top of the display list.
                proc draw {canvas tags x y width height orient color1 color2 args} {
                    # Ensure valid colors are provided while also converting to lists of 16-bit RGB values.
                    if { [catch {winfo rgb . $color1} rgb1] } {
                        error "invalid color: $color1"
                    }
                    if { [catch {winfo rgb . $color2} rgb2] } {
                        error "invalid color: $color2"
                    }
    
                    # Delete the gradient if it already exists.
                    $canvas delete [join $tags &&]
    
                    # Create a hidden canvas item to store meta data about this gradient.
                    set meta_data [list x $x  y $y  width $width  height $height  orient $orient  color1 $color1  color2 $color2  args $args]
                    $canvas create line $x $y $x $y  -state hidden  -tags [list {*}$tags meta [list gradient: {*}$meta_data]]
    
                    # Intercept the -animate option.
                    if { "-animate" in $args} {
                        set index [lsearch $args "-animate"]
                        set args  [lreplace $args $index $index]
                        set animate 1
                    } else {
                        set animate 0
                    }
    
                    # Draw the gradient.
                    if { $orient eq "central" } {
                        # Create the canvas rectangle items.
                        foreach {x1 y1 x2 y2 color} [get_rects $x $y $width $height $rgb1 $rgb2] {
                            $canvas create rectangle $x1 $y1 $x2 $y2  -outline $color  -width 2  -tags $tags  {*}$args
                            if {$animate} {update}
                        }
                    } elseif { $orient eq "circular" } {
                        # Create the canvas oval items.
                        foreach {x1 y1 x2 y2 color} [get_rects $x $y $width $height $rgb1 $rgb2] {
                            $canvas create oval $x1 $y1 $x2 $y2  -outline $color  -width 2  -tags $tags  {*}$args
                            if {$animate} {update}
                        }
                    } elseif { $orient eq "arc" } {
                        # Create the canvas arc items.
                        foreach {x1 y1 x2 y2 color} [get_rects $x $y $width $height $rgb1 $rgb2] {
                            $canvas create arc $x1 $y1 $x2 $y2  -outline $color  -width 2  -tags $tags  -style arc  {*}$args
                            if {$animate} {update}
                        }
                    } elseif { $orient in [list "vertical" "horizontal"] } {        
                        # Create the canvas line items.
                        foreach {x1 y1 x2 y2 color} [get_lines $x $y $width $height $orient $rgb1 $rgb2] {
                            $canvas create line $x1 $y1 $x2 $y2  -fill $color  -tags $tags  -capstyle projecting  {*}$args
                            if {$animate} {update}
                        }
                    } else {
                        # Delete the meta data and throw an error.
                        $canvas delete [join $tags &&]
                        error "invalid orientation: $orient; must be: vertical, horizontal, circular, central, or arc"
                    }
    
                    return
                }
    
                # Same as draw, but if the gradient already exists then it will retain its position in the canvas's display list after being redrawn.
                proc redraw {canvas tags x y width height orient color1 color2 args} {
                    # If the gradient already exists, remember its location in the display list.
                    set item_above [$canvas find above [join $tags &&]]
    
                    # Draw the gradient.
                    draw $canvas $tags $x $y $width $height $orient $color1 $color2 {*}$args
    
                    # Move the gradient to its previous location in the display list, if any.
                    if { $item_above ne "" } {
                        $canvas lower [join $tags &&] $item_above
                    }
    
                    return
                }
    
                # Shortcut procedure to change the colors of the gradient. Same as redraw but introspectively determines some meta data. 
                # This will use the gradient's x/y location when it was last drawn.
                proc recolor {canvas tags color1 color2} {
                    # Retrieve the gradient's meta data.
                    set meta [get_meta_data $canvas $tags]
                    set meta [dict remove $meta color1 color2]
                    dict with meta {}
    
                    # Redraw the gradient with the new colors.
                    redraw $canvas $tags $x $y $width $height $orient $color1 $color2 {*}$args
                    return     
                }
    
                # Shortcut procedure to resize and reposition a gradient. Same as redraw but introspectively determines some meta data. 
                proc resize {canvas tags x y width height} {
                    # Retrieve the gradient's meta data.
                    set meta [get_meta_data $canvas $tags]
                    set meta [dict remove $meta x y width height]
                    dict with meta {}
    
                    # Recreate the canvas lines with the new dimensions.
                    redraw $canvas $tags $x $y $width $height $orient $color1 $color2 {*}$args
                    return
                }
    
                # Retrieve meta information on a gradient.
                proc get_meta_data {canvas tags} {
                    if { [string is integer -strict $tags] } {
                        # This is supposedly a canvas item ID instead of a list of tags describing a gradient. Avoid introducing subtle logic errors.
                        error "cannot provide canvas item ID in lieu of gradient tags: $tags"
                    }
                    # This is a list of tags for a gradient. Find the hidden meta item for this gradient.
                    set all_tags [$canvas gettags "meta&&[join $tags &&]"]
                    set meta_data [lrange [lsearch -inline -index 0 $all_tags "gradient:"] 1 end]
                    if { [llength $meta_data] == 0 } {
                        error "cannot find meta data for gradient with tags: $tags"
                    }
                    return $meta_data
                }
    
                # For a canvas item ID, return all tags that are not gradient meta tags.
                proc get_non_meta_tags {canvas id} {
                    set tags [$canvas gettags $id]
                    set index [lsearch $tags "meta"]
                    set tags [lreplace $tags $index $index]
                    set index [lsearch -index 0 $tags "gradient:"]
                    set tags [lreplace $tags $index $index]
                    return $tags
                }
    
                # Calculate a list of center-to-outward colored rectangles suitable for [canvas create rectangle] or [canvas create oval].
                proc get_rects {x y width height rgb1 rgb2} {
                    set rects [list]
                    set x1 [expr {int($x)+1}]
                    set y1 [expr {int($y)+1}]
                    set x2 [expr {$x1+$width-1}]
                    set y2 [expr {$y1+$height-1}]
    
                    # Calculate each rectangle.
                    if { $width > $height } {
                        # Decrement X by one pixel in both directions. Decrement Y by a fraction in both directions.
                        # The Y axis will be more heavily concentrated.
                        set ratio  [expr { 1.0*$height/$width }]
                        set length [expr { $width/2 }]
                        while { $x2 > $x1 } {
                            lappend rects $x1 $y1 $x2 $y2 [get_color $rgb1 $rgb2 $length [expr { ($x2-$x1)/2 }]]
                            set x1 [expr { $x1+1 }]
                            set x2 [expr { $x2-1 }]
                            set y1 [expr { $y1+$ratio }]
                            set y2 [expr { $y2-$ratio }]
                        }
                    } else {
                        # Decrement Y by one pixel in both directions. Decrement X by a fraction in both directions.
                        # The X axis will be more heavily concentrated (unless width==height).
                        set ratio  [expr { 1.0*$width/$height }]
                        set length [expr { $height/2 }]
                        while { $y2 > $y1 } {
                            lappend rects $x1 $y1 $x2 $y2 [get_color $rgb1 $rgb2 $length [expr { ($y2-$y1)/2 }]]
                            set x1 [expr { $x1+$ratio }]
                            set x2 [expr { $x2-$ratio }]
                            set y1 [expr { $y1+1 }]
                            set y2 [expr { $y2-1 }]
                        }
                    }
                    return $rects
                }
    
                # Calculate a list of left-to-right or top-to-bottom colored lines suitable for [canvas create line].
                proc get_lines {x1 y1 width height orient rgb1 rgb2} {
                    set lines [list]
                    set x1 [expr {int($x1)}]
                    set y1 [expr {int($y1)}]
                    set x2 [expr {$x1+$width}]
                    set y2 [expr {$y1+$height}]
                    if { $orient eq "vertical" } {
                        # Calculate the color for each horizontal line.
                        for {set y $y1} {$y < $y2} {incr y} {
                            lappend lines $x1 $y $x2 $y [get_color $rgb1 $rgb2 $height [expr {$y-$y1}]]
                        }
                    } elseif { $orient eq "horizontal" } {
                        # Calculate the color for each vertical column.
                        for {set x $x1} {$x < $x2} {incr x} {
                            lappend lines $x $y1 $x $y2 [get_color $rgb1 $rgb2 $width [expr {$x-$x1}]]
                        }
                    } else {
                        error "invalid orientation: $orient; must be: vertical or horizontal"
                    }
    
                    return $lines
                }
    
                # Calculates the color at the specified index between rgb1 and rgb2 where rgb1 and rgb2 are the specified length apart.
                proc get_color {rgb1 rgb2 length index} {
                    # Throw an error if the index is out of bounds.
                    if { $index < 0  ||  $index >= $length } {
                        error "index $index is out of bounds for length $length"
                    }
    
                    lassign $rgb1 r1 g1 b1
                    lassign $rgb2 r2 g2 b2
    
                    # Determine the ratio between each starting component color and ending component color.
                    set r_ratio [expr { 1.00*($r2-$r1+1)/$length }]
                    set g_ratio [expr { 1.00*($g2-$g1+1)/$length }]
                    set b_ratio [expr { 1.00*($b2-$b1+1)/$length }]
    
                    # Calculate the new component colors at the given index. 
                    set r [expr { int($r_ratio*$index+$r1) }]
                    set g [expr { int($g_ratio*$index+$g1) }]
                    set b [expr { int($b_ratio*$index+$b1) }]
    
                    # A hacky workaround to make up for a lack of precision (or faulty math?).
                    # The final pixel of the gradient should exactly match the color of rgb2
                    if { $index == [expr {$length-1}] } {
                        lassign $rgb2 r g b
                    }
    
                    # Convert the integer RGB values to a hex color.
                    return [rgb_to_hex [list $r $g $b]]
                }
    
                # Convert a list of 16-bit RGB values to an 8-bit hex color. Using 8-bit hex colors instead of 16-bit
                # speeds up drawing of images two-fold (I haven't benchmarked this for canvases).
                proc rgb_to_hex {rgb} {
                    lassign $rgb r g b
                    set r [format %02x [expr {$r/256}]]
                    set g [format %02x [expr {$g/256}]]
                    set b [format %02x [expr {$b/256}]]
                    return #$r$g$b
                }
    
                # Converts 8-bit RGB values to 16-bit RGB values.
                proc rgb_8_to_16_bit {rgb} {
                    lassign $rgb r g b
                    return [list [expr {$r*256}] [expr {$g*256}] [expr {$b*256}]]
                }
    
                # Returns a random 8-bit hex color.
                proc random_color {} {
                    return [rgb_to_hex "[expr {int(rand()*65536)}] [expr {int(rand()*65536)}] [expr {int(rand()*65536)}]"]
                }
    
                # Redraw all gradients with random colors
                proc randomize_all_gradient_colors {canvas} {
                    foreach id [$canvas find withtag meta] {
                        set tags [get_non_meta_tags $canvas $id]
                        recolor $canvas $tags [random_color] [random_color]
                    }
                  return
                }
    
                # Randomize colors for all canvas items.
                proc randomize_all_canvas_item_colors {canvas} {
                    foreach id [$canvas find withtag all] {
                        if { [$canvas type $id] in {window} } {
                            continue
                        }
                        $canvas itemconfigure $id -fill [random_color]
                        if { [$canvas type $id] in {rectangle arc oval} } {
                             $canvas itemconfigure $id -outline [random_color]
                        }
                    }
                    return
                }
    
                # Run a demonstration.
                proc demo {} {
                    # Create a toplevel window with a canvas filling the contents.
                    set win .gradients
                    if { [winfo exists $win] } {
                        destroy $win
                    }
                    toplevel $win
                    set cvs [canvas $win.cvs -highlightthickness 0]
                    pack $cvs -fill both -expand yes
                    wm geometry $win 800x600
                    raise $win
    
                    # Fill the bottom of the screen with a striped green fade.
                    gradient draw $cvs footer 0 0 10 10 vertical white darkgreen -dash {3 1}
                    bind $cvs <Configure> {
                        gradient resize %W footer 0 [expr {%h-300}] %w 300
                    }
    
                    # Draw a strip of rainbow.
                    set width  100
                    set colors {white red orange yellow green blue purple violet white}
                    for {set i 1} {$i < [llength $colors]} {incr i} {
                        set c1 [lindex $colors $i-1]
                        set c2 [lindex $colors $i]
                        gradient draw $cvs "wave $i" [expr {$width*($i-1)}] 0 $width 50 horizontal $c1 $c2
                    }
    
                    # Draw some balls with diameters of 100.
                    set colors {green red yellow blue orange white}
                    for {set i 0} {$i < 6} {incr i} {
                        gradient draw $cvs "beachball $i" 0 55 100 100 arc [lindex $colors $i] #333 -start [expr {$i*60}] -extent 60
                    }
                    set colors {cyan pink orange firebrick}
                    for {set i 0} {$i < [llength $colors]} {incr i} {
                        gradient draw $cvs "quadball $i" 110 55 100 100 arc [lindex $colors $i] #eee -start [expr {$i*90}] -extent 90
                    }
                    gradient draw $cvs ball1    220 55 100 100 circular #bbbbbb white
                    gradient draw $cvs ball2    330 55 100 100 circular #888 #eee
                    gradient draw $cvs ball3    440 55 100 100 circular black grey
                    gradient draw $cvs ball4    550 55 100 100 circular black #555
                    gradient draw $cvs ball5    660 55 100 100 circular cyan black
    
                    # Draw some boxes.
                    gradient draw $cvs box1   0 160 150 100 horizontal white firebrick
                    gradient draw $cvs box2 150 160 200 100 central    black firebrick
                    gradient draw $cvs box3 350 160 100 100 horizontal firebrick black
                    gradient draw $cvs box4 450 160 200 100 central    firebrick black
                    gradient draw $cvs box5 650 160 150 100 horizontal black white
    
                    # Display some control buttons.
                    frame $cvs.f
                    button $cvs.f.b1 -text "Redraw with random gradient colors" -command [list [namespace current]::randomize_all_gradient_colors $cvs]
                    button $cvs.f.b2 -text "Relaunch demo with default colors" -command [list [namespace current]::demo]
                    button $cvs.f.b3 -text "Chaos" -command [list [namespace current]::randomize_all_canvas_item_colors $cvs]
                    pack $cvs.f.b1 $cvs.f.b2 $cvs.f.b3 -fill x
                    $cvs create window 400 400 -window $cvs.f
    
                    return
                }
            }
                    """
        )

    def recolor(self, canvas: Canvas, tags: str, color1, color2):
        """
        修改渐变图形颜色

        Args:
            canvas (Canvas): 被绘制的画布
            tags (str): 渐变图形标签
            color1 (str): 渐变颜色1
            color2 (str): 渐变颜色2
        """
        return self.tk.call(
            "gradient", "recolor", canvas._w,
            tags, color1, color2
        )

    def redraw(self,
               canvas: Canvas, tags: str,
               x: int, y: int, width: int, height: int,
               orient, color1, color2, **kw
               ):
        """
        绘制渐变图形，与draw相同

        Args:
            canvas (Canvas): 被绘制的画布
            tags (str): 渐变图形标签
            x (int): 渐变图形在画布上X轴坐标
            y (int): 渐变图形在画布上Y轴坐标
            width (int): 渐变图形的宽度
            height (int): 渐变图形的高度
            orient (str): 渐变图形的形状，有"arc" "central" "vertical" "horizontal" "circular"
            color1 (str): 渐变颜色1
            color2 (str): 渐变颜色2
        """
        kwargs = []
        for k in kw:
            kwargs.append("-" + k)
            kwargs.append(kw[k])
        return self.tk.call(
            "gradient", "redraw", canvas._w,
            tags, x, y, width, height, orient, color1, color2, *kwargs
        )

    def resize(self, canvas: Canvas, tags: str, x: int, y: int, width: int, height: int):
        """
        修改渐变图形位置

        Args:
            canvas (Canvas): 被绘制的画布
            tags (str): 渐变图形标签
            x (int): 渐变图形在画布上X轴坐标
            y (int): 渐变图形在画布上Y轴坐标
            width (int): 渐变图形的宽度
            height (int): 渐变图形的高度
        """
        return self.tk.call(
            "gradient", "recolor", canvas._w,
            tags, x, y, width, height
        )


X = "x"
Y = "y"


class Gradient2:
    def __init__(self):
        from tkinter import _default_root, Tk
        self.tk: Tk = _default_root
        self.init1()

    from tkinter import Canvas

    def demo1(self):
        self.tk.eval(
            """
  canvas .grad1
  bind .grad1 <Configure> [list drawGradient .grad1 x red royalblue]
  
  canvas .grad2
  bind .grad2 <Configure> [list drawGradient .grad2 y yellow red]
  
  pack .grad1 .grad2 -fill both -expand 1
            """
        )

    def draw1(self, win: Canvas, type, col1Str, col2Str):
        """
        用渐变颜色填充画布

        Args:
            win (Canvas): 被绘制的画布
            type (str): 渐变的方向，有"x" "y"
            col1Str (str): 渐变颜色1
            col2Str (str): 渐变颜色2

        """
        return self.tk.call("drawGradient", win._w, type, col1Str, col2Str)

    def init1(self):
        """
        初始化渐变模块
        参见：https://wiki.tcl-lang.org/page/Drawing+Gradients+on+a+Canvas
        """
        self.tk.eval(
            """
  proc + {n1 n2} {
    expr {$n1 + $n2}
  }
  proc - {n1 n2} {
    expr {$n1 - $n2}
  }
  proc * {n1 n2} {
    expr {$n1 * $n2}
  }
  proc / {n1 n2} {
    expr {$n1 / $n2}
  }
  proc toInt {n} {
    expr int($n)
  }
  
  proc drawGradient {win type col1Str col2Str} {
    $win delete gradient
    
    set width [winfo width $win]
    set height [winfo height $win]
    
    lassign [winfo rgb $win $col1Str] r1 g1 b1
    lassign  [winfo rgb $win $col2Str] r2 g2 b2
    set rRange [- $r2.0 $r1]
    set gRange [- $g2.0 $g1]
    set bRange [- $b2.0 $b1]
  
    if {$type == "x"} {
      set rRatio [/ $rRange $width]
      set gRatio [/ $gRange $width]
      set bRatio [/ $bRange $width]
    
      for {set x 0} {$x < $width} {incr x} {
        set nR [toInt [+ $r1 [* $rRatio $x]]]
        set nG [toInt [+ $g1 [* $gRatio $x]]]
        set nB [toInt [+ $b1 [* $bRatio $x]]]
  
        set col [format {%4.4x} $nR]
        append col [format {%4.4x} $nG]
        append col [format {%4.4x} $nB]
        $win create line $x 0 $x $height -tags gradient -fill #${col}
      }
    } else {
      set rRatio [/ $rRange $height]
      set gRatio [/ $gRange $height]
      set bRatio [/ $bRange $height]
  
      for {set y 0} {$y < $height} {incr y} {
        set nR [toInt [+ $r1 [* $rRatio $y]]]
        set nG [toInt [+ $g1 [* $gRatio $y]]]
        set nB [toInt [+ $b1 [* $bRatio $y]]]
  
        set col [format {%4.4x} $nR]
        append col [format {%4.4x} $nG]
        append col [format {%4.4x} $nB]
        $win create line 0 $y $width $y -tags gradient -fill #${col}
      }
    }
    return $win
  }
            """
        )


if __name__ == '__main__':
    from tkinter import Tk, Canvas

    root = Tk()

    canvas = Canvas()

    gradient = Gradient()
    gradient.draw(canvas, "gradient-rect-v",
                  5, 5, 50, 50, VERTICAL,
                  "blue", "red")
    gradient.draw(canvas, "gradient-rect-h",
                  60, 5, 50, 50, HORIZONTAL,
                  "blue", "red")
    gradient.draw(canvas, "gradient-rect-c",
                  115, 5, 50, 50, CENTRAL,
                  "blue", "red")
    gradient.draw(canvas, "gradient-arc",
                  170, 5, 50, 50, ARC,
                  "blue", "red", start=45, extent=135)
    gradient.draw(canvas, "gradient-circular",
                  225, 5, 50, 50, CIRCULAR,
                  "blue", "red")
    gradient.recolor(canvas, "gradient-rect-h", "blue", "black")

    canvas.pack(fill="both", expand="yes")

    canvas2 = Canvas()

    gradient2 = Gradient2()
    gradient2.draw1(canvas2, X, "blue", "red")

    canvas2.pack(fill="both", expand="yes")

    root.mainloop()



from tkadwold.layout.row import AdwLayoutRow, row_configure
from tkadwold.layout.column import AdwLayoutColumn, column_configure
from tkadwold.layout.put import AdwLayoutPut, put_configure
from tkadwold.layout.flow import Flow


class AdwLayout(AdwLayoutRow, AdwLayoutColumn, AdwLayoutPut, Flow):
    pass


from tkadw import *

根 = 主题主窗口(
    styles=["optimised"]
)
根.使用主题("win11", "light")
根.深黑模式(True)

按钮1 = 主题按钮(text="按钮1", command=lambda: print("Button1"))
按钮1.set_drawmode(0)
按钮1.pack(fill="both", expand="yes", padx=15, pady=(15, 7.5))

按钮2 = 主题按钮(text="按钮2", command=lambda: print("Button2"))
按钮2.configure(drawmode=1)
按钮2.pack(fill="both", expand="yes", padx=15, pady=(7.5, 15))

根.run()


from tkadwold import AdwDrawFrame


class AdwMenuBar(AdwDrawFrame):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def show(self):
        self.row(expand="no")



from tkadwold.windows.widgets.adw import Adw


class AgwWindow(Adw):
    def __init__(self, *args, resizable: bool = True, **kwargs):
        super().__init__(*args, **kwargs)
        self.resizable(resizable, resizable)

    def create_surface(self, *args, **kwargs):
        from tkadwold.game.surface import AgwSurface
        _surface = AgwSurface(self)
        _surface.show()
        return _surface

    def run(self):
        self.mainloop()


if __name__ == '__main__':
    window = AgwWindow()
    surface = window.create_surface()
    window.run()


import os.path
import os


def appconfig():
    if os.path.exists("app.config"):
        import configparser
        config = configparser.ConfigParser()
        config.read('app.config', encoding='UTF-8')

        if "title" in config["app"]:
            from tkinter import _default_root
            _default_root.title(config["app"]["title"])

        if "x" or "y" in config["app"]:
            from tkinter import _default_root

            pos = (int(config["app"]["x"]), int(config["app"]["y"]))

            _default_root.geometry(f"+{pos[0]}+{pos[1]}")

        if "width" or "height" in config["app"]:
            from tkinter import _default_root

            size = (int(config["app"]["width"]), int(config["app"]["height"]))

            _default_root.geometry(f"{size[0]}x{size[1]}")


if __name__ == '__main__':
    from tkadwold import Adwite
    root = Adwite()
    appconfig()
    root.mainloop()


from tkinterflow.tkinterflow import _flow, _flow_destroy


class Flow(object):

    mode = "place"

    def flow(self, mode="grid", *args, **kwargs):
        if mode == "place":
            self.mode = "place"
        else:
            self.mode = "grid"
        _flow(self, self.mode, *args, **kwargs)

    def flow_destroy(self):
        _flow_destroy(self, self.mode)


from tkinter import Widget


def put_configure(widget: Widget, x=0, y=0, width=0, height=0, top=None, bottom=None, left=None, right=None, *args, **kwargs):
    def configure(evt):
        if top is not None:
            y = top
        if left is not None:
            x = left
        if right is not None:
            width = widget.master.winfo_width() - x - right
        if bottom is not None:
            height = widget.master.winfo_height() -y -bottom
        widget.place(x=x, y=y, width=width, height=height, *args, **kwargs)
    configure(None)
    return widget.master.bind("<Configure>", configure, add="+"), widget.bind("<Configure>", configure, add="+"), \
        {"in": widget.place_info()["in"], "x": x, "y": y, "width": width, "height": height, "top": top, "bottom": bottom, "left": left, "right": right}


def put_forget(widget: Widget, putid):
    """
最好请使用Adw.run()
    """
    widget.place_forget()
    widget.unbind("<Configure>", putid)


class AdwLayoutPut:
    def put_configure(self, *args, **kwargs):
        r = put_configure(self, *args, **kwargs)
        self.putid = r[0]
        self.putid2 = r[1]
        self.putinfo = r[2]

    put = put_configure

    def put_forget(self):
        return put_forget(self, self.putid), put_forget(self, self.putid2)

    def put_info(self):
        return self.putinfo


class AdwLayout(Flow, AdwLayoutPut):
    pass


from tkadwold.windows.canvas.frame import AdwDrawFrame


class FluentFrame(AdwDrawFrame):
    def default_palette(self):
        self.palette_light()

    def palette_light(self):
        self.palette(
            {
                "frame": {
                    "back": "#f8f8f8",
                    "border": "#dbdbdb",
                    "border_width": 2,
                }
            }
        )

    def palette_dark(self):
        self.palette(
            {
                "frame": {
                    "back": "#050505",
                    "border": "#111111",
                    "border_width": 2,
                }
            }
        )


class FluentDarkFrame(FluentFrame):
    def default_palette(self):
        self.palette_dark()


if __name__ == '__main__':
    from tkadwold import Adw

    root = Adw()
    frame = FluentFrame()
    frame.pack(fill="both", expand="yes")
    root.mainloop()


from .base import AdwBase, Arg as AdwArgument
from .button import AdwButton
from .drawengine import AdwDrawEngine
from .drawwidget import AdwDrawWidget
from .entry import AdwEntry
from .gradient import Gradient as AdwGradient, Gradient2 as AdwGradient2, ARC, CENTRAL, CIRCULAR, HORIZONTAL, VERTICAL, X, Y
from .icon import ICONLIGHT,  ICONDARK
from .layout import AdwLayout
from .roundrect import RoundRect as AdwRoundRect
from .run import AdwRun, run
from .style import WindowStyle as AdwWindowStyle, LIGHT, DARK
from .theme import (
    AdwThemed, theme, theme_mode,
    AdwTMainWindow, AdwTButton, AdwTEntry
)
from .widget import AdwWidget
from .window import AdwMainWindow

# 中文版
from .base import AdwBase as 基本, Arg as 参数
from .button import AdwButton as 按钮
from .drawengine import AdwDrawEngine as 绘画引擎
from .drawwidget import AdwDrawWidget as 绘画组件
from .entry import AdwEntry as 输入框
from .gradient import Gradient as 渐变, Gradient2 as 渐变2, ARC as 弧形, CENTRAL as 中心, CIRCULAR as 圆形, HORIZONTAL as 水平, VERTICAL as 垂直, X as X坐标, Y as Y坐标
from .icon import ICONLIGHT,  ICONDARK
from .layout import AdwLayout as 布局
from .roundrect import RoundRect as 圆角矩形
from .run import AdwRun as 运行
from .style import WindowStyle as 窗口样式, LIGHT as 浅亮, DARK as 深黑
from .theme import (
    AdwThemed as 主题化, theme as 主题, theme_mode as 主题模式,
    AdwTMainWindow as 主题主窗口, AdwTButton as 主题按钮, AdwTEntry as 主题输入框
)
from .widget import AdwWidget as 小组件
from .window import AdwMainWindow as 主窗口



from tkadwold.windows.canvas.entry import AdwDrawEntry


class FluentEntry(AdwDrawEntry):
    def __init__(self, *args, width=160, height=50, **kwargs):
        super().__init__(*args, width=width, height=height, **kwargs)
        try:
            from tkextrafont import Font
            import os.path
            self.entry_text_font = Font(
                family="General Sans")
        except:
            from tkinter.font import families
            if "General Sans" in families():
                from tkinter.font import Font, names
                self.entry_text_font = Font(
                    family="General Sans")
        self.entry_text_font.configure(size=13)

    def _draw(self, evt):
        super()._draw(evt)

        self.create_gradient_v_rectangle(0,
                                         0,
                                         self._entry_border_width,
                                         self.winfo_height() / 2,
                                         "gradient_border_left_1",
                                         self._entry_border, self.entry_border2)

        self.create_gradient_v_rectangle(0,
                                         self.winfo_height() / 2,
                                         self._entry_border_width,
                                         self.winfo_height() / 2,
                                         "gradient_border_left_2",
                                         self.entry_border2, self._entry_border)

        self.create_gradient_v_rectangle(self.winfo_width()-self._entry_border_width,
                                         0,
                                         self._entry_border_width,
                                         self.winfo_height() / 2,
                                         "gradient_border_right_1",
                                         self._entry_border, self.entry_border2)

        self.create_gradient_v_rectangle(self.winfo_width()-self._entry_border_width,
                                         self.winfo_height() / 2,
                                         self._entry_border_width,
                                         self.winfo_height() / 2,
                                         "gradient_border_right_2",
                                         self.entry_border2, self._entry_border)

    def default_palette(self):
        self.palette_light()

    def palette_dark(self):
        self.palette(
            {
                "entry": {
                    "padding": (6, 6),

                    "back": "#080808",
                    "border": "#111111",
                    "border2": "#5f5f5f",
                    "text_back": "#cfcfcf",
                    "border_width": 2,

                    "bottom_line": "#0099bc",
                    "bottom_width": 0,

                    "focusin": {
                        "back": "#0D0D0D",
                        "border": "#111111",
                        "text_back": "#cfcfcf",
                        "border_width": 2,

                        "bottom_line": "#3bb1cc",
                        "bottom_width": 2,
                    }
                },
            }
        )

    def palette_light(self):
        self.palette(
            {
                "entry": {
                    "padding": (6, 6),

                    "back": "#f7f7f7",
                    "border": "#dbdbdb",
                    "border2": "#7e7e7e",
                    "text_back": "#000000",
                    "border_width": 2,

                    "bottom_line": "#0099bc",
                    "bottom_width": 0,

                    "focusin": {
                        "back": "#ececec",
                        "border": "#dbdbdb",
                        "text_back": "#000000",
                        "border_width": 2,

                        "bottom_line": "#3bb1cc",
                        "bottom_width": 2,
                    }
                },
            }
        )

    def palette(self, dict=None):
        super().palette(dict)
        if "entry" in dict:
            self.entry_border2 = dict["entry"]["border2"]


class FluentDarkEntry(FluentEntry):
    def default_palette(self):
        self.palette_dark()


if __name__ == '__main__':
    from tkadwold import Adw

    root = Adw()
    entry = FluentEntry()
    entry.pack(fill="both", expand="yes", padx=15, pady=15)
    root.mainloop()


LIGHT = "light"
DARK = "dark"


class WindowStyle:
    def __init__(self, master=None):
        """
        窗口样式
        _Windows_系统需安装_pywinstyles_库
        """
        if master is None:
            from tkinter import _default_root
            self.master = _default_root
        else:
            self.master = master

    def _style_mac(self, stylename: str):
        """
        设置MacOS平台下的窗口样式

        Args:
            stylename (str): 样式名称参见https://wiki.tcl-lang.org/page/MacWindowStyle

        """
        from os import system
        if stylename == "dark":
            system("defaults write -g NSRequiresAquaSystemAppearance -bool No")
        elif stylename == "light":
            system("defaults delete -g NSRequiresAquaSystemAppearance")
        else:
            self.master.call("::tk::unsupported::MacWindowStyle", "style", self.master._w, stylename)

    def _style_win(self, stylename: str):
        """
        设置Windows平台下的窗口样式

        Args:
            stylename (str): 样式名称参见https://pypi.org/project/pywinstyles
        """
        try:
            from pywinstyles import apply_style
            apply_style(self.master, stylename)
        except ModuleNotFoundError as error:
            print(error)

    def style(self, stylename: str):
        """
        设置窗口样式，暂时只支持Windows和MacOs平台

        Args:
            stylename (str): 样式名称，共有样式名_light_、_dark_
        """
        from sys import platform
        if platform == "win32" or platform == "cygwin":
            self._style_win(stylename)
        elif platform == "darwin":
            self._style_mac(stylename)


if __name__ == '__main__':
    from tkinter import Tk
    root = Tk()

    windowstyle = WindowStyle()
    windowstyle.style("dark")

    root.mainloop()


from tkadwold.windows.canvas.widget import AdwWidget


class AdwDrawBasicCheckBox(AdwWidget):
    def __init__(self, *args, width=30, height=30, **kwargs):
        super().__init__(*args, width=width, height=height, bd=0, highlightthickness=0, **kwargs)

        self.default_palette()

        self._checkbox_back = self.checkbox_back
        self._checkbox_border = self.checkbox_border
        self._checkbox_border_width = self.checkbox_border_width

        self.bind("<Button>", self._click, add="+")
        self.bind("<ButtonRelease>", self._unclick, add="+")
        self.bind("<Enter>", self._hover, add="+")
        self.bind("<Leave>", self._hover_release, add="+")

        self._draw(None)

    def _draw(self, evt):
        self.delete("all")

        self.checkbox_frame = self.create_rectangle(
            1.5, 1.5, self.winfo_width() - 3, self.winfo_height() - 3,
            width=self._checkbox_border_width,
            outline=self._checkbox_border, fill=self._checkbox_back,
        )

        self.checkbox_mark_line1 = self.create_line(self.winfo_x() + 5 + self.checkbox_mark_size,
                                                    self.winfo_y() + 10,
                                                    self.winfo_x() + 5 + self.checkbox_mark_size,
                                                    self.winfo_y() + 10 + self.checkbox_mark_size,
                                                    width=2, fill='black')

    def _click(self, evt=None):
        self.hover = True
        self._checkbox_back = self.checkbox_pressed_back
        self._checkbox_border = self.checkbox_pressed_border
        self._checkbox_border_width = self.checkbox_pressed_border_width

        self.focus_set()

        self._draw(None)

    def _unclick(self, evt=None):
        if self.hover:
            self._checkbox_back = self.checkbox_active_back
            self._checkbox_border = self.checkbox_active_border
            self._checkbox_border_width = self.checkbox_active_border_width

            self._draw(None)

            self.event_generate("<<Click>>")

    def _hover(self, evt=None):
        self.hover = True
        self._checkbox_back = self.checkbox_active_back
        self._checkbox_border = self.checkbox_active_border
        self._checkbox_border_width = self.checkbox_active_border_width

        self._draw(None)

    def _hover_release(self, evt=None):
        self.hover = False
        self._checkbox_back = self.checkbox_back
        self._checkbox_border = self.checkbox_border
        self._checkbox_border_width = self.checkbox_border_width

        self._draw(None)

    def default_palette(self):
        self.palette_light()

    def palette_light(self):
        self.palette(
            {
                "checkbox": {
                    "back": "#fdfdfd",
                    "border": "#eaeaea",
                    "border_width": 1,

                    "mark": {
                        "back": "#000000",
                        "size": 3
                    },

                    "active": {
                        "back": "#f9f9f9",
                        "border": "#aaaaaa",
                        "border_width": 1,
                        "checkbox_mark": "#eaeaea"
                    },

                    "pressed": {
                        "back": "#f9f9f9",
                        "border": "#e2e2e2",
                        "border_width": 1,
                        "checkbox_mark": "#000000"
                    },
                }
            }
        )

    def palette_dark(self):
        self.palette(
            {
                "button": {
                    "back": "#353535",
                    "border": "#454545",
                    "text_back": "#ffffff",
                    "border_width": 1,

                    "active": {
                        "back": "#3a3a3a",
                        "border": "#454545",
                        "text_back": "#cecece",
                        "border_width": 1,
                    },

                    "pressed": {
                        "back": "#2f2f2f",
                        "border": "#454545",
                        "text_back": "#9a9a9a",
                        "border_width": 1,
                    },
                }
            }
        )

    def palette(self, dict=None):
        if dict is not None:
            if "checkbox" in dict:
                self.checkbox_back = dict["checkbox"]["back"]
                self.checkbox_border = dict["checkbox"]["border"]
                self.checkbox_border_width = dict["checkbox"]["border_width"]

                if "mark" in dict["checkbox"]:
                    self.checkbox_mark_back = dict["checkbox"]["mark"]["back"]
                    self.checkbox_mark_size = dict["checkbox"]["mark"]["size"]

                if "active" in dict["checkbox"]:
                    self.checkbox_active_back = dict["checkbox"]["active"]["back"]
                    self.checkbox_active_border = dict["checkbox"]["active"]["border"]
                    self.checkbox_active_border_width = dict["checkbox"]["active"]["border_width"]

                    self.checkbox_active_mark = dict["checkbox"]["active"]["checkbox_mark"]

                if "pressed" in dict["checkbox"]:
                    self.checkbox_pressed_back = dict["checkbox"]["pressed"]["back"]
                    self.checkbox_pressed_border = dict["checkbox"]["pressed"]["border"]
                    self.checkbox_pressed_border_width = dict["checkbox"]["pressed"]["border_width"]

                    self.checkbox_pressed_mark = dict["checkbox"]["pressed"]["checkbox_mark"]

            self._palette = dict

            try:
                self._draw(None)
            except AttributeError:
                pass
        else:
            return self._palette


if __name__ == '__main__':
    from tkinter import Tk
    root = Tk()
    box = AdwDrawBasicCheckBox()
    box.pack()
    root.mainloop()


if __name__ == '__main__':
    from tkadwold.windows.theme import Adwite
    from tkadwold.designer.designer import AdwDesigner

    root = Adwite(default_theme="metro")

    designer = AdwDesigner(root)
    designer.row()

    root.animation()
    root.run()


from .drawengine import AdwDrawEngine


class AdwDrawWidget(AdwDrawEngine):
    def __init__(self, *args, width=80, height=30, **kwargs):
        super().__init__(*args, width=width, height=height, **kwargs)

        self.bind("<Configure>", self._event_configure, add="+")
        self.bind("<Enter>", self._event_enter, add="+")
        self.bind("<Leave>", self._event_leave, add="+")
        self.bind("<ButtonPress>", self._buttonpress, add="+")
        self.bind("<ButtonRelease>", self._buttonrelease, add="+")
        self.bind("<FocusIn>", self._event_focus_in, add="+")
        self.bind("<FocusOut>", self._event_focus_out, add="+")

        self._is_enter = False
        self._is_leave = True
        self._is_button = False
        self._is_focus = False

    def _buttonpress(self, event=None):
        self._is_button = True
        self.update()

    def _buttonrelease(self, event=None):
        if self._is_enter:
            self.event_generate("<<Click>>")
            self.focus_set()
        self._is_button = False
        self.update()

    def _event_configure(self, event=None):
        self.update()

    def _draw(self, event=None):
        """
        --请在此处绘制组件--
        """
        self.delete("all")

    def _event_enter(self, event=None):
        self._is_enter = True
        self._is_leave = False
        self.update()

    def _event_leave(self, event=None):
        self._is_enter = False
        self._is_leave = True
        self.update()

    def _event_focus_in(self, event=None):
        self._is_focus = True
        self.update()

    def _event_focus_out(self, event=None):
        self._is_focus = False
        self.update()

    def _other(self, event=None):
        self.configure(background=self.master.cget("background"))

    def default_palette(self):
        pass

    def palette(self):
        pass

    def update(self):
        super().update()
        self._other()
        self._draw()



from tkadwold.windows.fluent import *
from tkadwold import *

root = Adw()

frame = FluentFrame(root)

button = FluentButton(frame.frame, text="FluentButton")
button.pack(fill="x", padx=15, pady=15)

entry = FluentEntry(frame.frame)
entry.pack(fill="x", padx=15, pady=(0, 15))

frame.pack(fill="both", expand="yes", side="left")

frame2 = FluentDarkFrame(root)

button = FluentDarkButton(frame2.frame, text="FluentDarkButton")
button.pack(fill="x", padx=15, pady=15)

entry = FluentDarkEntry(frame2.frame)
entry.pack(fill="x", padx=15, pady=(0, 15))

frame2.pack(fill="both", expand="yes", side="right")

root.mainloop()


from .drawwidget import AdwDrawWidget


class AdwEntry(AdwDrawWidget):
    def __init__(self,
                 *args,
                 drawmode: int = 1,
                 text: str = "",
                 radius: int = 14,
                 show=...,
                 **kwargs):
        super().__init__(*args, **kwargs)

        self.args(
            drawmode=drawmode,
            back_default="#ffffff",
            back_focus="#ffffff",
            border_default="#e6e6e6",
            border_focus="#ebebeb",
            border_width_default=1,
            border_width_focus=1,
            bottomsheet_default="#9c9c9c",
            bottomsheet_focus="#005fb8",
            bottomsheet_width_default=1,
            bottomsheet_width_focus=2,
            radius=radius,
            text=text,
            text_default="#18191c",
            text_focus="#18191c",
        )

        self.default_palette()

        from tkinter import StringVar, Entry

        self.entryvar = StringVar(value=self._text)
        self.entrywidget = Entry(self, textvariable=self.entryvar, borderwidth=0, show=show, insertwidth=1)

    def _draw(self, event=None):
        super()._draw(event)
        if self._is_focus:
            __back = self._back_focus
            __border = self._border_focus
            __border_width = self._border_width_focus
            __bottomsheet = self._bottomsheet_focus
            __bottomsheet_width = self._bottomsheet_width_focus
            __text = self._text_focus
        else:
            __back = self._back_default
            __border = self._border_default
            __border_width = self._border_width_default
            __bottomsheet = self._bottomsheet_default
            __bottomsheet_width = self._bottomsheet_width_default
            __text = self._text_default
        try:
            self.entrywidget.configure(background=__back, foreground=__text, selectforeground=__text,
                                       insertbackground=__text)
        except:
            pass

        # 绘制框架
        if self._drawmode == 0:
            self.roundrect_draw(
                x=0, y=0,
                width=self.winfo_width(), height=self.winfo_height(),
                fill=__border, outline=__border, radius=self._radius + 2, tag="frame_border"
            )
            self._frame_border = "frame_border"
            self.roundrect_draw(
                x=__border_width, y=__border_width,
                width=self.winfo_width() - 2 * __border_width,
                height=self.winfo_height() - 2 * __border_width,
                fill=__back, outline=__back, radius=self._radius, tag="frame"
            )
        elif self._drawmode == 1:
            self.roundrect2_draw(
                x1=0, y1=0,
                x2=self.winfo_width() - __border_width,
                y2=self.winfo_height() - __border_width,
                fill=__back, outline=__border, radius=self._radius, tag="frame"
            )
        self._frame = "frame"

        self.entry_bottom = self.create_rectangle(__border_width + self._radius/3,
                                                  self.winfo_height() - __border_width + self._radius,
                                                  self.winfo_width() - __border_width - self._radius/3,
                                                  self.winfo_height() - __border_width,
                                                  fill=__bottomsheet, outline=__bottomsheet,
                                                  width=__bottomsheet_width)

        try:
            # 绘制输入组件
            self._entry = self.create_window(
                self.winfo_width() / 2, self.winfo_height() / 2,
                width=self.winfo_width() - self._radius - __border_width,
                height=self.winfo_height() - __border_width * 2 - 5,
                window=self.entrywidget
            )
        except AttributeError:
            pass

    def _event_focus_in(self, event=None):
        super()._event_focus_in()
        self.entrywidget.focus_set()

    def default_palette(self):
        pass

    def palette(self, palette: dict):
        if "entry" in palette:
            if "radius" in palette["entry"]:
                self._radius = palette["entry"]["radius"]
            if "default" in palette["entry"]:
                if "back" in palette["entry"]["default"]:
                    self._back_default = palette["entry"]["default"]["back"]
                if "border" in palette["entry"]["default"]:
                    self._border_default = palette["entry"]["default"]["border"]
                if "border_width" in palette["entry"]["default"]:
                    self._border_width_default = palette["entry"]["default"]["border_width"]
                if "bottomsheet" in palette["entry"]["default"]:
                    self._bottomsheet_default = palette["entry"]["default"]["bottomsheet"]
                if "bottomsheet_width" in palette["entry"]["default"]:
                    self._bottomsheet_width_default = palette["entry"]["default"]["bottomsheet_width"]
                if "fore" in palette["entry"]["default"]:
                    self._text_default = palette["entry"]["default"]["fore"]
            if "focus" in palette["entry"]:
                if "back" in palette["entry"]["focus"]:
                    self._back_focus = palette["entry"]["focus"]["back"]
                if "border" in palette["entry"]["focus"]:
                    self._border_focus = palette["entry"]["focus"]["border"]
                if "border_width" in palette["entry"]["focus"]:
                    self._border_width_focus = palette["entry"]["focus"]["border_width"]
                if "bottomsheet" in palette["entry"]["focus"]:
                    self._bottomsheet_focus = palette["entry"]["focus"]["bottomsheet"]
                if "bottomsheet_width" in palette["entry"]["focus"]:
                    self._bottomsheet_width_focus = palette["entry"]["focus"]["bottomsheet_width"]
                if "fore" in palette["entry"]["focus"]:
                    self._text_focus = palette["entry"]["focus"]["fore"]
        self.update()



from tkadwold.windows.canvas.button import AdwDrawButton


class FluentButton(AdwDrawButton):
    def __init__(self, *args, width=160, height=50, **kwargs):
        super().__init__(*args, width=width, height=height, **kwargs)
        try:
            from tkextrafont import Font
            import os.path
            self.button_text_font = Font(
                family="General Sans")
        except:
            from tkinter.font import families
            if "General Sans" in families():
                from tkinter.font import Font, names
                self.button_text_font = Font(
                    family="General Sans")
        self.button_text_font.configure(size=13)

    def _draw(self, evt):
        super()._draw(evt)

        self.create_gradient_v_rectangle(0,
                                         0,
                                         self._button_border_width,
                                         self.winfo_height() / 2,
                                         "gradient_border_left_1",
                                         self._button_border, self.button_border2)

        self.create_gradient_v_rectangle(0,
                                         self.winfo_height() / 2,
                                         self._button_border_width,
                                         self.winfo_height() / 2,
                                         "gradient_border_left_2",
                                         self.button_border2, self._button_border)

        self.create_gradient_v_rectangle(self.winfo_width()-self._button_border_width,
                                         0,
                                         self._button_border_width,
                                         self.winfo_height() / 2,
                                         "gradient_border_right_1",
                                         self._button_border, self.button_border2)

        self.create_gradient_v_rectangle(self.winfo_width()-self._button_border_width,
                                         self.winfo_height() / 2,
                                         self._button_border_width,
                                         self.winfo_height() / 2,
                                         "gradient_border_right_2",
                                         self.button_border2, self._button_border)

    def default_palette(self):
        self.palette_light()

    def palette_dark(self):
        self.palette(
            {
                "button": {
                    "back": "#080808",
                    "border": "#111111",
                    "border2": "#5f5f5f",
                    "text_back": "#cfcfcf",
                    "border_width": 2,

                    "active": {
                        "back": "#101010",
                        "border": "#111111",
                        "text_back": "#cfcfcf",
                        "border_width": 2,
                    },

                    "pressed": {
                        "back": "#0D0D0D",
                        "border": "#111111",
                        "text_back": "#cfcfcf",
                        "border_width": 2,
                    },
                }
            }
        )

    def palette_light(self):
        self.palette(
            {
                "button": {
                    "back": "#f7f7f7",
                    "border": "#dbdbdb",
                    "border2": "#7e7e7e",
                    "text_back": "#000000",
                    "border_width": 2,

                    "active": {
                        "back": "#dcdcdc",
                        "border": "#dbdbdb",
                        "text_back": "#000000",
                        "border_width": 2,
                    },

                    "pressed": {
                        "back": "#ececec",
                        "border": "#dbdbdb",
                        "text_back": "#000000",
                        "border_width": 2,
                    },
                }
            }
        )

    def palette(self, dict=None):
        super().palette(dict)
        if "button" in dict:
            self.button_border2 = dict["button"]["border2"]


class FluentDarkButton(FluentButton):
    def default_palette(self):
        self.palette_dark()


if __name__ == '__main__':
    from tkadwold import Adw

    root = Adw()
    button = FluentButton()
    button.pack(fill="both", expand="yes", padx=15, pady=15)
    root.mainloop()


from tkinter import Pack

START = "start"
END = "end"


def row_configure(widget: Pack, side="top", fill="both", expand="yes", anchor="center", padx=0, pady=0, ipadx=0, ipady=0, *args, **kwargs):
    widget.pack(
        side=side, fill=fill, expand=expand, anchor=anchor,
        padx=padx, pady=pady, ipadx=ipadx, ipady=ipady,
        *args, **kwargs
    )


class AdwLayoutRow:
    def row_configure(self, *args, **kwargs):
        row_configure(self, *args, **kwargs)

    row = row_configure

    def row_forget(self):
        self.pack_forget()

    def row_info(self):
        return self.pack_info()


if __name__ == '__main__':
    from tkadwold import Adwite, AdwTButton

    class TestButton(AdwTButton):
        def __init__(self, *args, **kwargs):
            super().__init__(*args, command=self.destroy, **kwargs)

    root = Adwite()
    for index in range(5):
        TestButton(root, text=index, width=40, height=40).row(padx=5, pady=5)
    root.mainloop()


from tkinter import Event, Widget, Tk, Frame


x, y = 0, 0


def click(event: Event):
    global x, y
    x, y = event.x, event.y


def move(event: Event, widget: Widget, window: Tk):
    global x, y
    new_x = (event.x - x) + window.winfo_x()
    new_y = (event.y - y) + window.winfo_y()
    if new_y <= 0:
        new_y = 0
    window.geometry(f"+{new_x}+{new_y}")


def bind_move(widget: Widget, window: Tk):
    widget.bind("<Button-1>", click)
    widget.bind("<B1-Motion>", lambda event: move(event, widget, window))


def tag_bind_move(widget: Widget, tag, window: Tk):
    widget.tag_bind(tag, "<Button-1>", click)
    widget.tag_bind(tag, "<B1-Motion>", lambda event: move(event, widget, window))


class DragArea(object):
    def __init__(self, widget: Widget = Frame, master: Widget = None, window: Tk = None, *args, **kwargs):
        self._widget = widget(master, args, **kwargs)

        if window is None:
            from tkinter import _default_root

            window: widget = _default_root

        bind_move(self._widget, window)

        if hasattr(self._widget, "children"):
            for index in self._widget.children:
                try:
                    bind_move(index, window)
                except:
                    tag_bind_move(self._widget, index, window)

        if hasattr(self._widget, "frame"):
            bind_move(self._widget.frame, window)

        if hasattr(self._widget, "frame_f"):
            tag_bind_move(self._widget, self._widget.frame_f, window)

        if hasattr(self._widget, "frame_frame"):
            tag_bind_move(self._widget, self._widget.frame_f, window)

    @property
    def widget(self) -> Widget:
        return self._widget


if __name__ == '__main__':
    from tkadwold import Adwite, AdwTFrame, AdwTButton

    root = Adwite(default_theme="win11")
    root.geometry("200x250")
    root.titlebar(False)

    area = DragArea(AdwTFrame, root, root, height=40)
    area.widget.pack(fill="x", side="top")

    button = AdwTButton(area.widget, text="X", height=30, width=30, command=lambda: root.quit())
    button.pack(fill="y", side="right", padx=3, pady=3)

    root.mainloop()


from tkinter import Pack

START = "start"
END = "end"


def column_configure(widget: Pack, fill="both", expand="yes", anchor="center", padx=0, pady=0, ipadx=0, ipady=0):
    widget.pack(side="left", fill=fill, expand=expand, anchor=anchor,
                padx=padx, pady=pady, ipadx=ipadx, ipady=ipady)


class AdwLayoutColumn:
    def column_configure(self, *args, **kwargs):
        column_configure(self, *args, **kwargs)

    column = column_configure

    def column_forget(self):
        self.pack_forget()

    def column_info(self):
        return self.pack_info()


if __name__ == '__main__':
    from tkadwold import Adwite, AdwTButton

    class TestButton(AdwTButton):
        def __init__(self, *args, **kwargs):
            super().__init__(*args, command=self.destroy, **kwargs)

    root = Adwite()
    for index in range(5):
        w = TestButton(root, text=index, width=40, height=40)
        w.column(padx=5, pady=5)
        print(w.column_info())
    root.mainloop()


import typing

win11_theme = {
    "light": {
        "window": {
            "back": "#f4f4f4"
        },

        "label": {
            "back": "transparent",
            "text_back": "#000000",
        },

        "button": {
            "radius": 13,
            "back": "#fdfdfd",
            "border": "#ededed",
            "text_back": "#202020",
            "border_width": 1,

            "active": {
                "back": "#f9f9f9",
                "border": "#d5d5d5",
                "text_back": "#202020",
                "border_width": 1,
            },

            "pressed": {
                "back": "#fafafa",
                "border": "#ebebeb",
                "text_back": "#202020",
                "border_width": 1,
            },
        },

        "circular_button": {
            "back": "#fdfdfd",
            "border": "#ededed",
            "text_back": "#202020",
            "border_width": 1,

            "active": {
                "back": "#f9f9f9",
                "border": "#d5d5d5",
                "text_back": "#202020",
                "border_width": 1,
            },

            "pressed": {
                "back": "#fafafa",
                "border": "#ebebeb",
                "text_back": "#202020",
                "border_width": 1,
            },
        },

        "frame": {
            "radius": 15,
            "back": "#fafafa",
            "border": "#e7e7e7",
            "border_width": 1,
            "padding": 0
        },

        "entry": {
            "radius": 13,
            "padding": (8, 8),

            "back": "#ffffff",
            "border": "#e6e6e6",
            "text_back": "#18191c",
            "border_width": 1,

            "bottom_line": "#9c9c9c",
            "bottom_width": 1,

            "focusin": {
                "back": "#ffffff",
                "border": "#ebebeb",
                "text_back": "#18191c",
                "border_width": 1,

                "bottom_line": "#005fb8",
                "bottom_width": 2,
            }
        },

        "text": {
            "radius": 13,
            "padding": (8, 8),

            "back": "#ffffff",
            "border": "#e6e6e6",
            "text_back": "#18191c",
            "border_width": 1,

            "bottom_line": "#9c9c9c",
            "bottom_width": 1,

            "focusin": {
                "back": "#ffffff",
                "border": "#ebebeb",
                "text_back": "#18191c",
                "border_width": 1,

                "bottom_line": "#005fb8",
                "bottom_width": 2,
            }
        },

        "separator": {
            "back": "#d0d0d0",
            "border_width": 1,

            "rounded": True
        },

        "designer_frame": {
            "border": "black",
            "grip": "black"
        },

        "listbox": {
            "back": "#fafafa",
            "border": "#e7e7e7",
            "border_width": 1,
            "padding": 0,
            "radius": 6,

            "item_back": "#fafafa",
            "item_border": "#fafafa",
            "item_text": "#000000",

            "item_radius": 10,
            "item_height": 36,
            "item_padding": 1,

            "active": {
                "item_back": "#005fb8",
                "item_border": "#005fb8",
                "item_text": "#ffffff",
            },
        }
    },
    "dark": {
        "window": {
            "back": "#202020"
        },

        "label": {
            "back": "transparent",
            "text_back": "#ffffff",
        },

        "button": {
            "radius": 13,
            "back": "#2a2a2a",
            "border": "#313131",
            "text_back": "#ebebeb",
            "border_width": 1,

            "active": {
                "back": "#2f2f2f",
                "border": "#313131",
                "text_back": "#ebebeb",
                "border_width": 1,
            },

            "pressed": {
                "back": "#232323",
                "border": "#2c2c2c",
                "text_back": "#ebebeb",
                "border_width": 1,
            },
        },

        "circular_button": {
            "back": "#2a2a2a",
            "border": "#313131",
            "text_back": "#ebebeb",
            "border_width": 1,

            "active": {
                "back": "#2f2f2f",
                "border": "#313131",
                "text_back": "#ebebeb",
                "border_width": 1,
            },

            "pressed": {
                "back": "#232323",
                "border": "#2c2c2c",
                "text_back": "#ebebeb",
                "border_width": 1,
            },
        },

        "frame": {
            "radius": 15,
            "back": "#1c1c1c",
            "border": "#2f2f2f",
            "border_width": 1,
            "padding": 0
        },

        "entry": {
            "radius": 13,
            "padding": (8, 8),

            "back": "#2c2c2c",
            "border": "#383838",
            "text_back": "#e7e9eb",
            "border_width": 1,

            "bottom_line": "#686868",
            "bottom_width": 1,

            "focusin": {
                "back": "#1c1c1c",
                "border": "#2c2c2c",
                "text_back": "#e7e9eb",
                "border_width": 1,

                "bottom_line": "#57c8ff",
                "bottom_width": 2,
            }
        },

        "text": {
            "radius": 13,
            "padding": (8, 8),

            "back": "#2c2c2c",
            "border": "#383838",
            "text_back": "#e7e9eb",
            "border_width": 1,

            "bottom_line": "#686868",
            "bottom_width": 1,

            "focusin": {
                "back": "#1c1c1c",
                "border": "#2c2c2c",
                "text_back": "#e7e9eb",
                "border_width": 1,

                "bottom_line": "#57c8ff",
                "bottom_width": 2,
            }
        },

        "separator": {
            "back": "#404040",
            "border_width": 1,

            "rounded": True
        },

        "designer_frame": {
            "border": "white",
            "grip": "white"
        },

        "listbox": {
            "back": "#1c1c1c",
            "border": "#2f2f2f",
            "border_width": 1,
            "padding": 0,
            "radius": 6,

            "item_back": "#1c1c1c",
            "item_border": "#1c1c1c",
            "item_text": "#ffffff",

            "item_radius": 10,
            "item_height": 36,
            "item_padding": 1,

            "active": {
                "item_back": "#57c8ff",
                "item_border": "#57c8ff",
                "item_text": "#000000",
            },
        }
    }
}

gtk_theme = {
    "light": {
        "window": {
            "back": "#ffffff"
        },

        "label": {
            "back": "transparent",
            "text_back": "#000000",
        },

        "button": {
            "radius": 11,
            "back": "#f6f5f4",
            "border": "#ccc6c1",
            "text_back": "#2e3436",
            "border_width": 1.3,

            "active": {
                "back": "#f8f8f7",
                "border": "#dad6d2",
                "text_back": "#2e3436",
                "border_width": 1.3,
            },

            "pressed": {
                "back": "#dad6d2",
                "border": "#dad6d2",
                "text_back": "#2e3436",
                "border_width": 1.3,
            },
        },

        "circular_button": {
            "back": "#f6f5f4",
            "border": "#ccc6c1",
            "text_back": "#2e3436",
            "border_width": 1.3,

            "active": {
                "back": "#f8f8f7",
                "border": "#dad6d2",
                "text_back": "#2e3436",
                "border_width": 1.3,
            },

            "pressed": {
                "back": "#dad6d2",
                "border": "#dad6d2",
                "text_back": "#2e3436",
                "border_width": 1.3,
            },
        },

        "frame": {
            "radius": 0,
            "back": "#f6f5f4",
            "border": "#d5d0cc",
            "border_width": 1,
            "padding": 0
        },

        "entry": {
            "radius": 11,
            "padding": (5, 5),

            "back": "#ffffff",
            "border": "#cdc7c2",
            "text_back": "#000000",
            "border_width": 1,

            "bottom_line": "#eaeaea",
            "bottom_width": 0,

            "focusin": {
                "back": "#ffffff",
                "border": "#3584e4",
                "text_back": "#000000",
                "border_width": 1,

                "bottom_line": "#185fb4",
                "bottom_width": 0,
            }
        },

        "text": {
            "padding": (5, 5),

            "radius": 0,
            "back": "#ffffff",
            "border": "#cdc7c2",
            "text_back": "#000000",
            "border_width": 1,

            "bottom_line": "#eaeaea",
            "bottom_width": 0,

            "focusin": {
                "back": "#ffffff",
                "border": "#3584e4",
                "text_back": "#000000",
                "border_width": 1,

                "bottom_line": "#185fb4",
                "bottom_width": 0,
            }
        },

        "separator": {
            "back": "#d8d4d0",
            "border_width": 1,

            "rounded": True
        },

        "designer_frame": {
            "border": "black",
            "grip": "black"
        },

        "listbox": {
            "back": "#f6f5f4",
            "border": "#d5d0cc",
            "border_width": 1,
            "padding": 0,
            "radius": 0,

            "item_back": "#f6f5f4",
            "item_border": "#f6f5f4",
            "item_text": "#000000",

            "item_radius": 8,
            "item_height": 25,
            "item_padding": 1,

            "active": {
                "item_back": "#3584e4",
                "item_border": "#3584e4",
                "item_text": "#ffffff",
            },
        }
    },
    "dark": {
        "window": {
            "back": "#353535"
        },

        "label": {
            "back": "transparent",
            "text_back": "#ffffff",
        },

        "button": {
            "radius": 11,
            "back": "#353535",
            "border": "#1b1b1b",
            "text_back": "#eeeeec",
            "border_width": 1.3,

            "active": {
                "back": "#373737",
                "border": "#1b1b1b",
                "text_back": "#eeeeec",
                "border_width": 1.3,
            },

            "pressed": {
                "back": "#1e1e1e",
                "border": "#282828",
                "text_back": "#eeeeec",
                "border_width": 1.3,
            },
        },

        "circular_button": {
            "back": "#353535",
            "border": "#1b1b1b",
            "text_back": "#eeeeec",
            "border_width": 1.3,

            "active": {
                "back": "#373737",
                "border": "#1b1b1b",
                "text_back": "#eeeeec",
                "border_width": 1.3,
            },

            "pressed": {
                "back": "#1e1e1e",
                "border": "#282828",
                "text_back": "#eeeeec",
                "border_width": 1.3,
            },
        },

        "frame": {
            "radius": 0,
            "back": "#313131",
            "border": "#1b1b1b",
            "border_width": 1,
            "padding": 0
        },

        "entry": {
            "radius": 11,
            "padding": (5, 5),

            "back": "#2d2d2d",
            "border": "#1f1f1f",
            "text_back": "#cccccc",
            "border_width": 1,

            "bottom_line": "#eaeaea",
            "bottom_width": 0,

            "focusin": {
                "back": "#2d2d2d",
                "border": "#3584e4",
                "text_back": "#ffffff",
                "border_width": 1,

                "bottom_line": "#3584e4",
                "bottom_width": 0,
            }
        },

        "text": {
            "padding": (5, 5),

            "radius": 0,
            "back": "#2d2d2d",
            "border": "#1f1f1f",
            "text_back": "#cccccc",
            "border_width": 1,

            "bottom_line": "#eaeaea",
            "bottom_width": 0,

            "focusin": {
                "back": "#2d2d2d",
                "border": "#3584e4",
                "text_back": "#ffffff",
                "border_width": 1,

                "bottom_line": "#3584e4",
                "bottom_width": 0,
            }
        },

        "separator": {
            "back": "#282828",
            "border_width": 1,

            "rounded": True
        },

        "designer_frame": {
            "border": "white",
            "grip": "white"
        },

        "listbox": {
            "back": "#313131",
            "border": "#1b1b1b",
            "border_width": 1,
            "padding": 0,
            "radius": 0,

            "item_back": "#313131",
            "item_border": "#313131",
            "item_text": "#ffffff",

            "item_radius": 8,
            "item_height": 25,
            "item_padding": 1,

            "active": {
                "item_back": "#3584e4",
                "item_border": "#3584e4",
                "item_text": "#000000",
            },
        },
    }

}

bilibili_theme = {
    "light": {
        "window": {
            "back": "#ffffff"
        },

        "label": {
            "back": "transparent",
            "text_back": "#000000",
        },

        "button": {
            "radius": 16,
            "back": "#ffffff",
            "border": "#e3e5e7",
            "text_back": "#18191c",
            "border_width": 1,

            "active": {
                "back": "#e3e5e7",
                "border": "#e3e5e7",
                "text_back": "#18191c",
                "border_width": 1,
            },

            "pressed": {
                "back": "#eceff0",
                "border": "#e3e5e7",
                "text_back": "#6d7479",
                "border_width": 1,
            },
        },

        "circular_button": {
            "back": "#ffffff",
            "border": "#e3e5e7",
            "text_back": "#18191c",
            "border_width": 1,

            "active": {
                "back": "#e3e5e7",
                "border": "#e3e5e7",
                "text_back": "#18191c",
                "border_width": 1,
            },

            "pressed": {
                "back": "#eceff0",
                "border": "#e3e5e7",
                "text_back": "#6d7479",
                "border_width": 1,
            },
        },

        "frame": {
            "radius": 18,
            "back": "#f6f7f8",
            "border": "#f1f2f3",
            "border_width": 1,
            "padding": 0
        },

        "entry": {
            "radius": 16,
            "padding": (8, 8),

            "back": "#ffffff",
            "border": "#e3e5e7",
            "text_back": "#18191c",
            "border_width": 1,

            "bottom_line": "#eaeaea",
            "bottom_width": 0,

            "focusin": {
                "back": "#ffffff",
                "border": "#ff6699",
                "text_back": "#18191c",
                "border_width": 1,

                "bottom_line": "#185fb4",
                "bottom_width": 0,
            }
        },

        "text": {
            "radius": 16,
            "padding": (8, 8),

            "back": "#ffffff",
            "border": "#e3e5e7",
            "text_back": "#18191c",
            "border_width": 1,

            "bottom_line": "#eaeaea",
            "bottom_width": 0,

            "focusin": {
                "back": "#ffffff",
                "border": "#ff6699",
                "text_back": "#18191c",
                "border_width": 1,

                "bottom_line": "#185fb4",
                "bottom_width": 0,
            }
        },

        "separator": {
            "back": "#f1f2f3",
            "border_width": 1,

            "rounded": True
        },

        "designer_frame": {
            "border": "#ff6699",
            "grip": "#ff6699"
        },

        "listbox": {
            "back": "#f6f7f8",
            "border": "#f1f2f3",
            "border_width": 1,
            "padding": 0,
            "radius": 18,

            "item_back": "#f6f7f8",
            "item_border": "#f6f7f8",
            "item_text": "#000000",

            "item_radius": 14,
            "item_height": 40,
            "item_padding": 1,

            "active": {
                "item_back": "#d44e7d",
                "item_border": "#d44e7d",
                "item_text": "#ffffff",
            },
        }, },

    "dark": {
        "window": {
            "back": "#17181a"
        },

        "label": {
            "back": "transparent",
            "text_back": "#ffffff",
        },

        "button": {
            "radius": 16,
            "back": "#242628",
            "border": "#2f3134",
            "text_back": "#dcdee0",
            "border_width": 1,

            "active": {
                "back": "#2f3134",
                "border": "#2f3134",
                "text_back": "#dcdee0",
                "border_width": 1,
            },

            "pressed": {
                "back": "#26282a",
                "border": "#2f3134",
                "text_back": "#a9abad",
                "border_width": 1,
            },
        },

        "circular_button": {
            "radius": 16,
            "back": "#242628",
            "border": "#2f3134",
            "text_back": "#dcdee0",
            "border_width": 1,

            "active": {
                "back": "#2f3134",
                "border": "#2f3134",
                "text_back": "#dcdee0",
                "border_width": 1,
            },

            "pressed": {
                "back": "#26282a",
                "border": "#2f3134",
                "text_back": "#a9abad",
                "border_width": 1,
            },
        },

        "frame": {
            "radius": 18,
            "back": "#1e2022",
            "border": "#232527",
            "border_width": 1,
            "padding": 0
        },

        "entry": {
            "radius": 16,
            "padding": (8, 8),

            "back": "#17181a",
            "border": "#2f3134",
            "text_back": "#e7e9eb",
            "border_width": 1,

            "bottom_line": "#eaeaea",
            "bottom_width": 0,

            "focusin": {
                "back": "#17181a",
                "border": "#d44e7d",
                "text_back": "#e7e9eb",
                "border_width": 1,

                "bottom_line": "#d44e7d",
                "bottom_width": 0,
            }
        },

        "text": {
            "radius": 16,
            "padding": (8, 8),

            "back": "#17181a",
            "border": "#2f3134",
            "text_back": "#e7e9eb",
            "border_width": 1,

            "bottom_line": "#eaeaea",
            "bottom_width": 0,

            "focusin": {
                "back": "#17181a",
                "border": "#d44e7d",
                "text_back": "#e7e9eb",
                "border_width": 1,

                "bottom_line": "#d44e7d",
                "bottom_width": 0,
            }
        },

        "separator": {
            "back": "#232527",
            "border_width": 1,

            "rounded": True
        },

        "designer_frame": {
            "border": "#ff6699",
            "grip": "#ff6699"
        },

        "listbox": {
            "back": "#1e2022",
            "border": "#232527",
            "border_width": 1,
            "padding": 0,
            "radius": 18,

            "item_back": "#1e2022",
            "item_border": "#1e2022",
            "item_text": "#ffffff",

            "item_radius": 14,
            "item_height": 40,
            "item_padding": 1,

            "active": {
                "item_back": "#d44e7d",
                "item_border": "#d44e7d",
                "item_text": "#000000",
            },
        },
    }
}

metro_theme = {
    "light": {
        "window": {
            "back": "#ffffff"
        },

        "label": {
            "back": "transparent",
            "text_back": "#000000",
        },

        "button": {
            "radius": 0,
            "back": "#eeeeee",
            "border": "#cccccc",
            "text_back": "#000000",
            "border_width": 1,

            "active": {
                "back": "#666666",
                "border": "#666666",
                "text_back": "#ffffff",
                "border_width": 1,
            },

            "pressed": {
                "back": "#333333",
                "border": "#333333",
                "text_back": "#ffffff",
                "border_width": 1,
            },
        },

        "circular_button": {
            "back": "#eeeeee",
            "border": "#cccccc",
            "text_back": "#000000",
            "border_width": 1,

            "active": {
                "back": "#666666",
                "border": "#666666",
                "text_back": "#ffffff",
                "border_width": 1,
            },

            "pressed": {
                "back": "#333333",
                "border": "#333333",
                "text_back": "#ffffff",
                "border_width": 1,
            },
        },

        "frame": {
            "radius": 0,
            "back": "#ffffff",
            "border": "#bfbfbf",
            "border_width": 1,
            "padding": 0
        },

        "entry": {
            "radius": 0,
            "padding": (3, 3),

            "back": "#ffffff",
            "border": "#999999",
            "text_back": "#000000",
            "border_width": 1,

            "bottom_line": "#ffffff",
            "bottom_width": 0,

            "focusin": {
                "back": "#ffffff",
                "border": "#00aedb",
                "text_back": "#000000",
                "border_width": 1,

                "bottom_line": "#ffffff",
                "bottom_width": 0,
            }
        },

        "text": {
            "radius": 0,
            "padding": (3, 3),

            "back": "#ffffff",
            "border": "#999999",
            "text_back": "#000000",
            "border_width": 1,

            "bottom_line": "#ffffff",
            "bottom_width": 0,

            "focusin": {
                "back": "#ffffff",
                "border": "#00aedb",
                "text_back": "#000000",
                "border_width": 1,

                "bottom_line": "#ffffff",
                "bottom_width": 0,
            }
        },

        "separator": {
            "back": "#999999",
            "border_width": 1,

            "rounded": True
        },

        "designer_frame": {
            "border": "black",
            "grip": "black"
        },

        "listbox": {
            "back": "#ffffff",
            "border": "#bfbfbf",
            "border_width": 1,
            "padding": 0,
            "radius": 0,

            "item_back": "#ffffff",
            "item_border": "#ffffff",
            "item_text": "#000000",

            "item_radius": 8,
            "item_height": 35,
            "item_padding": 1,

            "active": {
                "item_back": "#666666",
                "item_border": "#666666",
                "item_text": "#ffffff",
            },
        },
    },
    "dark": {
        "window": {
            "back": "#111111"
        },

        "label": {
            "back": "transparent",
            "text_back": "#ffffff",
        },

        "button": {
            "radius": 0,
            "back": "#222222",
            "border": "#444444",
            "text_back": "#ffffff",
            "border_width": 1,

            "active": {
                "back": "#aaaaaa",
                "border": "#aaaaaa",
                "text_back": "#000000",
                "border_width": 1,
            },

            "pressed": {
                "back": "#eeeeee",
                "border": "#eeeeee",
                "text_back": "#000000",
                "border_width": 1,
            },
        },

        "circular_button": {
            "back": "#222222",
            "border": "#444444",
            "text_back": "#ffffff",
            "border_width": 1,

            "active": {
                "back": "#aaaaaa",
                "border": "#aaaaaa",
                "text_back": "#000000",
                "border_width": 1,
            },

            "pressed": {
                "back": "#eeeeee",
                "border": "#eeeeee",
                "text_back": "#000000",
                "border_width": 1,
            },
        },

        "frame": {
            "radius": 0,
            "back": "#111111",
            "border": "#373737",
            "border_width": 1,
            "padding": 0
        },

        "entry": {
            "radius": 0,
            "padding": (3, 3),

            "back": "#111111",
            "border": "#999999",
            "text_back": "#ffffff",
            "border_width": 1,

            "bottom_line": "#000000",
            "bottom_width": 0,

            "focusin": {
                "back": "#111111",
                "border": "#00aedb",
                "text_back": "#ffffff",
                "border_width": 1,

                "bottom_line": "#000000",
                "bottom_width": 0,
            }
        },

        "text": {
            "radius": 0,
            "padding": (3, 3),

            "back": "#111111",
            "border": "#999999",
            "text_back": "#ffffff",
            "border_width": 1,

            "bottom_line": "#000000",
            "bottom_width": 0,

            "focusin": {
                "back": "#111111",
                "border": "#00aedb",
                "text_back": "#ffffff",
                "border_width": 1,

                "bottom_line": "#000000",
                "bottom_width": 0,
            }
        },

        "separator": {
            "back": "#999999",
            "border_width": 1,

            "rounded": False
        },

        "designer_frame": {
            "border": "white",
            "grip": "white"
        },

        "listbox": {
            "back": "#111111",
            "border": "#373737",
            "border_width": 1,
            "padding": 0,
            "radius": 0,

            "item_back": "#111111",
            "item_border": "#111111",
            "item_text": "#ffffff",

            "item_radius": 8,
            "item_height": 35,
            "item_padding": 1,

            "active": {
                "item_back": "#aaaaaa",
                "item_border": "#aaaaaa",
                "item_text": "#000000",
            },
        },
    }
}

pypi_org_theme = {
    "light": {
        "window": {
            "back": "#ffffff"
        },

        "label": {
            "back": "transparent",
            "text_back": "#000000",
        },

        "button": {
            "radius": 8,
            "back": "#006dad",
            "border": "#006dad",
            "text_back": "#ffffff",
            "border_width": 1,

            "active": {
                "back": "#00507f",
                "border": "#00507f",
                "text_back": "#ffffff",
                "border_width": 1,
            },

            "pressed": {
                "back": "#00507f",
                "border": "#ffffff",
                "text_back": "#ffffff",
                "border_width": 1,
            },
        },

        "circular_button": {
            "back": "#006dad",
            "border": "#006dad",
            "text_back": "#ffffff",
            "border_width": 1,

            "active": {
                "back": "#00507f",
                "border": "#00507f",
                "text_back": "#ffffff",
                "border_width": 1,
            },

            "pressed": {
                "back": "#00507f",
                "border": "#ffffff",
                "text_back": "#ffffff",
                "border_width": 1,
            },
        },

        "frame": {
            "radius": 0,
            "back": "#ececec",
            "border": "#d3d3d3",
            "border_width": 1,
            "padding": 0
        },

        "entry": {
            "radius": 8,
            "padding": (5, 5),

            "back": "#ffffff",
            "border": "#949494",
            "text_back": "#464646",
            "border_width": 1,

            "bottom_line": "#ffffff",
            "bottom_width": 0,

            "focusin": {
                "back": "#ffffff",
                "border": "#006dad",
                "text_back": "#464646",
                "border_width": 3,

                "bottom_line": "#ffffff",
                "bottom_width": 0,
            }
        },

        "text": {
            "radius": 8,
            "padding": (8, 8),

            "back": "#ffffff",
            "border": "#949494",
            "text_back": "#464646",
            "border_width": 1,

            "bottom_line": "#ffffff",
            "bottom_width": 0,

            "focusin": {
                "back": "#ffffff",
                "border": "#006dad",
                "text_back": "#464646",
                "border_width": 3,

                "bottom_line": "#ffffff",
                "bottom_width": 0,
            }
        },

        "separator": {
            "back": "#d3d3d3",
            "border_width": 1,

            "rounded": False
        },

        "designer_frame": {
            "border": "#006dad",
            "grip": "#006dad"
        },

        "listbox": {
            "back": "#ececec",
            "border": "#d3d3d3",
            "border_width": 1,
            "padding": 0,
            "radius": 0,

            "item_back": "#ececec",
            "item_border": "#ececec",
            "item_text": "#000000",

            "item_radius": 0,
            "item_height": 35,
            "item_padding": 0,

            "active": {
                "item_back": "#00507f",
                "item_border": "#00507f",
                "item_text": "#ffffff",
            },
    },
    },

    "dark": {
        "window": {
            "back": "#000002"
        },

        "label": {
            "back": "transparent",
            "text_back": "#ffffff",
        },

        "button": {
            "radius": 8,
            "back": "#00569f",
            "border": "#00569f",
            "text_back": "#ffffff",
            "border_width": 1,

            "active": {
                "back": "#00356c",
                "border": "#00356c",
                "text_back": "#ffffff",
                "border_width": 1,
            },

            "pressed": {
                "back": "#00356c",
                "border": "#000000",
                "text_back": "#ffffff",
                "border_width": 1,
            },
        },

        "circular_button": {
            "back": "#00569f",
            "border": "#00569f",
            "text_back": "#ffffff",
            "border_width": 1,

            "active": {
                "back": "#00356c",
                "border": "#00356c",
                "text_back": "#ffffff",
                "border_width": 1,
            },

            "pressed": {
                "back": "#00356c",
                "border": "#000000",
                "text_back": "#ffffff",
                "border_width": 1,
            },
        },

        "frame": {
            "radius": 0,
            "back": "#0c1012",
            "border": "#30373b",
            "border_width": 1,
            "padding": 0
        },

        "entry": {
            "radius": 8,
            "padding": (5, 5),

            "back": "#000000",
            "border": "#1e2427",
            "text_back": "#d1deca",
            "border_width": 1,

            "bottom_line": "#ffffff",
            "bottom_width": 0,

            "focusin": {
                "back": "#000000",
                "border": "#004d8f",
                "text_back": "#d1deca",
                "border_width": 3,

                "bottom_line": "#ffffff",
                "bottom_width": 0,
            }
        },

        "text": {
            "radius": 8,
            "padding": (5, 5),

            "back": "#000000",
            "border": "#1e2427",
            "text_back": "#d1deca",
            "border_width": 1,

            "bottom_line": "#ffffff",
            "bottom_width": 0,

            "focusin": {
                "back": "#000000",
                "border": "#004d8f",
                "text_back": "#d1deca",
                "border_width": 3,

                "bottom_line": "#ffffff",
                "bottom_width": 0,
            }
        },

        "separator": {
            "back": "#30373b",
            "border_width": 1,

            "rounded": False
        },

        "designer_frame": {
            "border": "#00569f",
            "grip": "#00569f"
        },

        "listbox": {
            "back": "#0c1012",
            "border": "#30373b",
            "border_width": 1,
            "padding": 0,
            "radius": 0,

            "item_back": "#0c1012",
            "item_border": "#0c1012",
            "item_text": "#ffffff",

            "item_radius": 0,
            "item_height": 35,
            "item_padding": 0,

            "active": {
                "item_back": "#00356c",
                "item_border": "#00356c",
                "item_text": "#000000",
            },
        },
    }
}


def _set_theme_mode(mode):
    from os import environ
    environ["tkAdwite.ThemeMode"] = mode


def set_theme_mode(mode="system"):
    if mode == "system":
        try:
            from darkdetect import isDark
            if isDark():
                _set_theme_mode("dark")
            else:
                _set_theme_mode("light")
        except ModuleNotFoundError:
            pass
    else:
        _set_theme_mode(mode)


def get_theme_mode():
    from os import environ
    if "tkAdwite.ThemeMode" in environ:
        return environ["tkAdwite.ThemeMode"]
    else:
        return "light"


def get_default_theme():
    from os import environ
    from json import loads
    try:
        return loads(environ["tkAdwite.DefaultTheme"])
    except KeyError:
        return None


def _set_default_theme(theme):
    from os import environ
    from json import dumps
    environ["tkAdwite.DefaultTheme"] = dumps(theme)


def set_default_theme(theme, mode="system"):
    if theme == "win11" or theme == "windows11" or theme == "Windows11" or theme == "Win11":
        _set_default_theme(win11_theme)
    elif theme == "gtk" or theme == "Gtk":
        _set_default_theme(gtk_theme)
    elif theme == "bilibili" or theme == "BiliBili":
        _set_default_theme(bilibili_theme)
    elif theme == "metro" or theme == "Metro":
        _set_default_theme(metro_theme)
    elif theme == "pypi" or theme == "Pypi":
        _set_default_theme(pypi_org_theme)
    else:
        _set_default_theme(theme)


set_default_theme("win11")

from tkadwold.windows.widgets.adw import Adw


def check_system_toggle_theme(callable: typing.Callable[[str], None]) -> None:
    try:
        import threading
        import darkdetect
    except ModuleNotFoundError:
        pass
    else:
        t = threading.Thread(target=darkdetect.listener, args=(callable,))
        t.daemon = True
        t.start()


class Adwite(Adw):
    def __init__(self, default_theme="win11", default_theme_mode="system", *args, **kwargs):
        super().__init__(*args, **kwargs)
        check_system_toggle_theme(self.system_switch_theme)
        self.set_default_theme_mode(default_theme_mode)
        self.set_default_theme(default_theme)

    def system_switch_theme(self, theme):
        self.event_generate("<<SystemThemeSwitch>>", data=theme)

    def get_system_theme(self) -> str:
        try:
            from darkdetect import theme
        except ModuleNotFoundError:
            pass
        else:
            return theme()

    def get_json(self):
        from json import dumps
        return dumps(
            {
                "root": {
                    "type": type(self).__name__,
                    "title": self.title()
                }
            }
        )

    def set_default_theme(self, theme, _mode="auto"):
        """
        win11、basic、metro、bilibili、gtk
        """
        set_default_theme(theme, _mode)
        for widget in self.winfo_children():
            if hasattr(widget, "palette"):
                widget.palette(get_default_theme())
                widget.update()
                if hasattr(widget, "frame"):
                    for widgetf in widget.frame.winfo_children():
                        if hasattr(widgetf, "palette"):
                            widgetf.palette(get_default_theme())
                            widgetf.update()
        self.palette(get_default_theme()[get_theme_mode()])

        from sys import platform
        """
        if platform == "win32":
            from sys import platform
            from ctypes import windll, byref, sizeof, c_int
            from ctypes.wintypes import RGB
            color = self.winfo_rgb(self.cget("bg"))
            windll.dwmapi.DwmSetWindowAttribute(windll.user32.GetParent(self.winfo_id()), 35,
                                                byref(c_int(RGB(color[0], color[1], color[2]))),
                                                sizeof(c_int))
        """

    def set_default_theme_mode(self, mode):
        set_theme_mode(mode)
        for widget in self.winfo_children():
            if hasattr(widget, "palette"):
                widget.update()
                widget.palette(get_default_theme())
            if hasattr(widget, "frame"):
                for widgetf in widget.frame.winfo_children():
                    if hasattr(widgetf, "palette"):
                        widgetf.update()
                        widgetf.palette(get_default_theme())
        self.palette(get_default_theme()[get_theme_mode()])

    def default_palette(self):
        self.palette(get_default_theme()[get_theme_mode()])


from tkadwold.windows.widgets.toplevel import AdwToplevel


class Adwitew(Adwite, AdwToplevel):
    pass


class AdwThemed(object):
    def default_palette(self):
        self.palette(get_default_theme()[get_theme_mode()])

    def palette(self, *args, **kwargs):
        super().palette(get_default_theme()[get_theme_mode()])
        for child in self.winfo_children():
            if hasattr(child, "palette"):
                child.palette(get_default_theme()[get_theme_mode()])
        if hasattr(self, "frame"):
            for child in self.frame.winfo_children():
                if hasattr(child, "palette"):
                    child.palette(get_default_theme()[get_theme_mode()])


from tkadwold.windows.widgets import AdwLabel


class AdwTLabel(AdwThemed, AdwLabel):
    pass


from tkadwold.windows.canvas.button import AdwDrawRoundButton3


class AdwTButton(AdwThemed, AdwDrawRoundButton3):
    def win11_accent_dark(self):
        self.palette(
            {
                "button": {
                    "radius": 13,
                    "back": "#57c8ff",
                    "border": "#64cdff",
                    "text_back": "#1c1c1c",
                    "border_width": 1,

                    "active": {
                        "back": "#51b7eb",
                        "border": "#5fbced",
                        "text_back": "#1c1c1c",
                        "border_width": 1,
                    },

                    "pressed": {
                        "back": "#4ba6d5",
                        "border": "#59aed9",
                        "text_back": "#1c1c1c",
                        "border_width": 1,
                    },
                }
            }
        )

    def win11_accent_light(self):
        self.palette(
            {
                "button": {
                    "radius": 13,
                    "back": "#0560b6",
                    "border": "#1a6cba",
                    "text_back": "#ebebeb",
                    "border_width": 1,

                    "active": {
                        "back": "#1e6fbc",
                        "border": "#307bc2",
                        "text_back": "#ebebeb",
                        "border_width": 1,
                    },

                    "pressed": {
                        "back": "#327ec5",
                        "border": "#4288ca",
                        "text_back": "#ebebeb",
                        "border_width": 1,
                    },
                }
            }
        )


from tkadwold.windows.canvas.button import AdwDrawCircularButton


class AdwTCircularButton(AdwThemed, AdwDrawCircularButton):
    pass


from tkadwold.windows.canvas.entry import AdwDrawRoundEntry3


class AdwTEntry(AdwThemed, AdwDrawRoundEntry3):
    pass


from tkadwold.designer.designerframe import AdwDesignerFrame


class AdwTDesignerFrame(AdwThemed, AdwDesignerFrame):
    pass


from tkadwold.windows.canvas.textbox import AdwDrawRoundText3


class AdwTText(AdwThemed, AdwDrawRoundText3):
    pass


from tkadwold.windows.canvas.frame import AdwDrawRoundFrame3


class AdwTFrame(AdwThemed, AdwDrawRoundFrame3):
    pass


from tkadwold.windows.widgets.listbox import AdwRoundListBox3


class AdwTListBox(AdwThemed, AdwRoundListBox3):
    pass


from tkadwold.windows.canvas.separator import AdwDrawSeparator


class AdwTSeparator(AdwThemed, AdwDrawSeparator):
    pass


class AdwTStack(AdwTFrame):
    def __init__(self, *args, **kwargs):
        """
        用于切换不同的界面
        """
        super().__init__(*args, **kwargs)
        self._pages = {}

    def add_page(self, page: AdwTFrame, id: int = 0):
        """
        添加页面

        :param page: 页面组件
        :param id: 组件ID
        """
        self._pages[id] = page

    def show_page(self, id: int):
        """
        显示页面，会将其他页面隐藏

        :param id: 被显示的页面ID
        """
        self._pages[id].pack(fill="both", expand="yes")
        for item in self._pages.keys():
            if not item == id:
                self.hide_page(item)

    def hide_page(self, id: int):
        """
        内置函数，最好不要使用，因为几乎没有什么用
        """
        self._pages[id].pack_forget()

    def get_page(self, id: int):
        """
        获取页面

        :param id: 所要获取的页面ID
        """
        return self._pages[id]

    def get_pages(self):
        """
        获取所有页面
        """
        return self._pages


from tkadwold.windows.widgets.mdi import AdwMDI


class AdwTMDI(AdwThemed, AdwMDI):
    def create_child(self, x=10, y=10, width=150, height=150, can_close: bool = True, content=AdwTFrame,
                     closebutton=AdwTCircularButton, childframe=AdwTFrame, titlebarframe=AdwTFrame
                     ):
        return super().create_child(
            x=x, y=y, width=width, height=height, can_close=can_close, content=content,
            closebutton=closebutton, childframe=childframe, titlebarframe=titlebarframe
        )

    def create_designer_child(self, x=10, y=10, width=150, height=150, can_close: bool = True, content=AdwTFrame,
                              closebutton=AdwTCircularButton, childframe=AdwTFrame, titlebarframe=AdwTFrame
                              ):
        return super().create_designer_child(
            x=x, y=y, width=width, height=height, can_close=can_close, content=content,
            closebutton=closebutton, childframe=childframe, titlebarframe=titlebarframe
        )


class AdwTTabs(AdwTFrame):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.tab_panel = AdwTListBox(self.frame, item_selected=self.switch_page)
        self.tab_pages = AdwTStack(self.frame)

        self.tab_panel.column(expand="no")
        self.tab_pages.column()

    def switch_page(self):
        self.tab_pages.show_page(self.tab_panel.select())

    def add(self, page, text: str = ""):
        i = self.tab_panel.cget("items")
        i.append(text)
        self.tab_panel.configure(items=i)
        self.tab_pages.add_page(page, text)
        self.tab_pages.update()
        self.tab_panel.update()

    def destroy(self):
        self.tab_panel.destroy()
        self.tab_pages.destroy()
        super().destroy()


if __name__ == '__main__':
    root = Adwite(default_theme="win11")
    root.geometry("600x300")
    root.style_dark(True)


    def command():
        widgets_panel.show_page(choose_panel.select())


    def switch_theme(event):
        print(event)
        root.set_default_theme("bilibili", root.get_system_theme())


    root.bind("<<SystemThemeSwitch>>", switch_theme)

    choose_panel = AdwTListBox(
        items=sorted(
            ["AdwTButton", "AdwTCircularButton", "AdwTDesignerFrame", "AdwTEntry", "AdwTMDI", "AdwTTabs"]
        ),
        item_selected=command
    )
    choose_panel.column(expand=False, padx=(15, 5), pady=15, ipadx=5, ipady=5)

    widgets_panel = AdwTStack()
    widgets_panel.column(padx=(5, 15), pady=15, ipadx=5, ipady=5)

    button_panel = AdwTFrame(widgets_panel.frame)
    button = AdwTButton(button_panel.frame, text="AdwTButton")
    button.row(expand="no")
    widgets_panel.add_page(button_panel, "AdwTButton")

    cbutton_panel = AdwTFrame(widgets_panel.frame)
    cbutton = AdwTCircularButton(cbutton_panel.frame, text="AdwTCircularButton")
    cbutton.row(expand="no", fill="none")
    widgets_panel.add_page(cbutton_panel, "AdwTCircularButton")

    designerframe_panel = AdwTFrame(widgets_panel.frame)
    designerframe = AdwTDesignerFrame(designerframe_panel.frame, widget=AdwTButton, text="AdwTDesignerFrame")
    designerframe.place(x=5, y=5, width=160, height=60)
    widgets_panel.add_page(designerframe_panel, "AdwTDesignerFrame")

    entry_panel = AdwTFrame(widgets_panel.frame)
    entry = AdwTEntry(entry_panel.frame)
    entry.row(expand="no")
    widgets_panel.add_page(entry_panel, "AdwTEntry")

    mdi_panel = AdwTFrame(widgets_panel.frame)
    mdi = AdwTMDI(mdi_panel.frame)
    mdi_child1 = mdi.create_child()
    mdi.row()
    widgets_panel.add_page(mdi_panel, "AdwTMDI")

    tabs = AdwTTabs(widgets_panel.frame)
    for i in range(5):
        f = AdwTFrame(tabs.tab_pages.frame)
        l = AdwTLabel(f.frame, text=i)
        l.row()
        tabs.add(f, f"AdwTTab {i}")
    tabs.row()
    widgets_panel.add_page(tabs, "AdwTTabs")

    choose_panel.select("AdwTButton")
    widgets_panel.show_page("AdwTButton")

    root.mainloop()



from tkadwold.windows.canvas.drawengine import AdwDrawEngine


class AgwSurface(AdwDrawEngine):
    def __init__(self, window, background="#000000"):
        super().__init__(window, highlightthickness=0, background=background)

    def show(self):
        self.pack(fill="both", expand="yes")




from tkadwold.windows.canvas.frame import AdwDrawFrame, AdwDrawRoundFrame, AdwDrawRoundFrame3
from tkadwold.windows.canvas.drawengine import AdwDrawEngine


class _Item:
    def __init__(self, canvas: AdwDrawEngine, listbox, item, i1, i2, iactive, irelease):
        self.canvas = canvas
        self.listbox = listbox
        self.item = item
        self.i1 = i1
        self.i2 = i2
        self.iactive = iactive
        self.irelease = irelease
        i = """
        self.canvas.tag_bind(i1, "<Button>", lambda evt: self.active(evt, item))
        self.canvas.tag_bind(i1, "<ButtonRelease>", lambda evt: self.release(evt, item))
        self.canvas.tag_bind(i2, "<Button>", lambda evt: self.active(evt, item))
        self.canvas.tag_bind(i2, "<ButtonRelease>", lambda evt: self.release(evt, item))
        """

        if self.listbox.select() == item:
            self.canvas.itemconfigure(
                self.i1, outline=self.iactive[1], fill=self.iactive[0]
            )
            self.canvas.itemconfigure(
                self.i2, fill=self.iactive[2]
            )

        it = """
        self.canvas.tag_bind(i1, "<Button>", self._click, add="+")
        self.canvas.tag_bind(i1, "<ButtonRelease>", self._unclick, add="+")
        self.canvas.tag_bind(i1, "<Enter>", self._hover, add="+")
        self.canvas.tag_bind(i1, "<Leave>", self._hover_release, add="+")
        """

        self.canvas.tag_bind(i1, "<Button>", self._click2, add="+")
        self.canvas.tag_bind(i2, "<Button>", self._click2, add="+")

    def _click2(self, evt=None):
        self.listbox.select(self.item)

    def _click(self, evt=None):
        self.hover = True

    def _unclick(self, evt=None):
        if self.hover:
            self.listbox.select(self.item)

    def _hover(self, evt=None):
        self.hover = True

    def _hover_release(self, evt=None):
        self.hover = False

    def click(self, event, item):
        self.canvas.itemconfigure(
            self.i1, outline=self.iactive[1], fill=self.iactive[0]
        )
        self.canvas.itemconfigure(
            self.i2, fill=self.iactive[2]
        )

    def release(self, event, item):
        self.canvas.itemconfigure(
            self.i1, outline=self.irelease[1], fill=self.irelease[0]
        )
        self.canvas.itemconfigure(
            self.i2, fill=self.irelease[2]
        )


class AdwListBox(AdwDrawFrame):
    def __init__(self, *args, items: list = [], item_selected=None, item_repeat_selected=None, **kwargs):
        self.itemrepeatselected = None
        super().__init__(*args, **kwargs)
        self.selectitem = None

        self.configure(items=items, item_repeat_selected=item_repeat_selected, item_selected=item_selected)

        self.item_canvas = AdwDrawEngine(self.frame, bd=0, highlightthickness=0)
        self.item_canvas.pack(fill="both", expand="yes", padx=3, pady=3)

    def select(self, item=None):
        if item is None:
            return self.selectitem
        else:
            #print(item)
            self.focus_set()
            if item == self.selectitem:
                if self.itemrepeatselected is not None:
                    self.itemrepeatselected()
            self.selectitem = item
            self.event_generate("<<ItemSelected>>")
            if self.itemselected is not None:
                self.itemselected()
            self._draw(None)

    def configure(self, **kwargs):
        if "items" in kwargs:
            self._items = kwargs.pop("items")
        if "item_selected" in kwargs:
            self.itemselected = kwargs.pop("item_selected")
        if "item_repeat_selected" in kwargs:
            self.itemrepeatselected = kwargs.pop("item_repeat_selected")
        else:
            super().configure(**kwargs)

    def cget(self, key):
        if key == "items":
            return self._items
        else:
            return super().cget(key)

    def _draw(self, evt):
        super()._draw(evt)
        try:
            def release(event, item):
                pass

            self.item_canvas.delete("all")
            self.item_canvas.configure(background=self.frame_back)
            _i = 0
            self.items = {}
            for item in self._items:
                i1 = self.item_canvas.create_rectangle(
                    self.item_padding,
                    _i * self.item_height + self.item_padding,
                    self.item_canvas.winfo_width()-self.item_padding,
                    (_i + 1) * self.item_height,
                    outline=self.item_border, fill=self.item_back
                )
                i2 = self.item_canvas.create_text(
                    0+self.item_padding+5,
                    _i * self.item_height + self.item_padding + self.item_height/2,
                    text=item, anchor="w", fill=self.item_text
                )
                _i += 1

                i3 = _Item(self.item_canvas, self, item, i1, i2,
                           [self.item_active_back, self.item_active_border, self.item_active_text],
                           [self.item_back, self.item_border, self.item_text], )

                self.items[item] = [i1, i2]

        except:
            pass

    def palette(self, dict=None):
        if dict is not None:
            if "listbox" in dict:
                self.frame_back = dict["listbox"]["back"]
                self.frame_border = dict["listbox"]["border"]
                self.frame_border_width = dict["listbox"]["border_width"]
                self.frame_padding = dict["listbox"]["padding"]

                if "radius" in dict["listbox"]:
                    self.frame_radius = dict["listbox"]["radius"]

                self.item_back = dict["listbox"]["item_back"]
                self.item_border = dict["listbox"]["item_border"]
                self.item_text = dict["listbox"]["item_text"]

                if "item_radius" in dict["listbox"]:
                    self.item_radius = dict["listbox"]["item_radius"]

                self.item_active_back = dict["listbox"]["active"]["item_back"]
                self.item_active_border = dict["listbox"]["active"]["item_border"]
                self.item_active_text = dict["listbox"]["active"]["item_text"]

                self.item_height = dict["listbox"]["item_height"]
                self.item_padding = dict["listbox"]["item_padding"]

            self._palette = dict

            try:
                self._draw(None)
            except AttributeError:
                pass
        else:
            return self._palette

    def palette_light(self):
        self.palette(
            {
                "listbox": {
                    "back": "#ffffff",
                    "border": "#eaeaea",
                    "border_width": 2,
                    "padding": 0,
                    "radius": 3,

                    "item_back": "#ffffff",
                    "item_border": "#eaeaea",
                    "item_text": "#000000",

                    "item_radius": 6,
                    "item_height": 40,
                    "item_padding": 3,

                    "active": {
                        "item_back": "#185fb4",
                        "item_border": "#185fb4",
                        "item_text": "#ffffff",
                    },
                }
            }
        )

    def palette_dark(self):
        self.palette(
            {
                "listbox": {
                    "back": "#0f0f0f",
                    "border": "#333333",
                    "border_width": 2,
                    "padding": 0,
                    "radius": 3,

                    "item_back": "#0f0f0f",
                    "item_border": "#333333",
                    "item_text": "#ffffff",

                    "item_radius": 6,
                    "item_height": 40,
                    "item_padding": 3,

                    "active": {
                        "item_back": "#4cc2ff",
                        "item_border": "#4cc2ff",
                        "item_text": "#000000",
                    },
                }
            }
        )


class AdwRoundListBox2(AdwListBox, AdwDrawRoundFrame):
    def __init__(self, *args, items: list = [], **kwargs):
        super().__init__(*args, items=items, **kwargs)

    def _draw(self, evt):
        super()._draw(evt)
        try:
            def release(event, item):
                pass

            self.item_canvas.delete("all")
            self.item_canvas.configure(background=self.frame_back)
            _i = 0
            self.items = {}
            for item in self._items:
                i1 = self.item_canvas.create_round_rect2(
                    self.item_padding,
                    _i * self.item_height + self.item_padding,
                    self.item_canvas.winfo_width()-self.item_padding,
                    (_i + 1) * self.item_height,
                    radius=self.frame_radius,
                    outline=self.item_border, fill=self.item_back
                )
                i2 = self.item_canvas.create_text(
                    0+self.item_padding+5,
                    _i * self.item_height + self.item_padding + self.item_height/2,
                    text=item, anchor="w", fill=self.item_text
                )
                _i += 1

                i3 = _Item(self.item_canvas, self, item, i1, i2,
                           [self.item_active_back, self.item_active_border, self.item_active_text],
                           [self.item_back, self.item_border, self.item_text], )

                self.items[item] = [i1, i2]

        except:
            pass


class AdwRoundListBox3(AdwListBox, AdwDrawRoundFrame3):
    def __init__(self, *args, items: list = [], **kwargs):
        super().__init__(*args, items=items, **kwargs)

    def default_palette(self):
        self.palette(
            {
                "listbox": {
                    "back": "#ffffff",
                    "border": "#eaeaea",
                    "border_width": 2,
                    "padding": 0,
                    "radius": 8,

                    "item_back": "#ffffff",
                    "item_border": "#eaeaea",
                    "item_text": "#000000",

                    "item_radius": 10,
                    "item_height": 40,
                    "item_padding": 3,

                    "active": {
                        "item_back": "#185fb4",
                        "item_border": "#185fb4",
                        "item_text": "#ffffff",
                    },
                }
            }
        )

    def _draw(self, evt):
        super()._draw(evt)
        try:
            def release(event, item):
                pass

            self.item_canvas.delete("all")
            self.item_canvas.configure(background=self.frame_back)
            _i = 0
            self.items = {}
            for item in self._items:
                i1 = self.item_canvas.create_round_rect4(
                    self.item_padding,
                    _i * self.item_height + self.item_padding,
                    self.item_canvas.winfo_width()-self.item_padding,
                    (_i + 1) * self.item_height,
                    radius=self.item_radius,
                    outline=self.item_border, fill=self.item_back
                )
                i2 = self.item_canvas.create_text(
                    0+self.item_padding+5,
                    _i * self.item_height + self.item_padding + self.item_height/2,
                    text=item, anchor="w", fill=self.item_text
                )
                _i += 1

                i3 = _Item(self.item_canvas, self, item, i1, i2,
                           [self.item_active_back, self.item_active_border, self.item_active_text],
                           [self.item_back, self.item_border, self.item_text], )

                self.items[item] = [i1, i2]

        except:
            pass


if __name__ == '__main__':
    from tkadwold import Adw, Adwite, pypi_org_dark_theme

    root = Adw()
    #root = Adwite()
    #root.set_default_theme("Pypi", "dark")

    listbox = AdwRoundListBox3()
    #listbox.palette(pypi_org_dark_theme)

    listbox.configure(items=["Hello", "World"])

    listbox.row(padx=5, pady=5)

    root.mainloop()


from tkinter import Tk, _get_default_root
from json import dumps, loads


class AdwBuilder(object):
    def __init__(self, data=None, data_of_json=None):
        _ = data
        if data_of_json is not None:
            _ = loads(data_of_json)
        if "adwite" in _:
            if "theme" in _["adwite"]:
                from tkadwold import set_default_theme
                mode = "system"
                if "mode" in _["adwite"]:
                    mode = _["adwite"]["mode"]
                set_default_theme(_["adwite"]["theme"], mode)
        self.keys = {}
        if "widgets" in _:
            widgets = _["widgets"]
        elif "Widgets" in _:
            widgets = _["Widgets"]
        else:
            widgets = None
        if widgets is not None:
            for index in widgets:
                _name = index.split("::")[0]
                _type = index.split("::")[1]
                if _type == "Adwite":
                    from tkadwold.windows.theme import Adwite
                    type = Adwite
                elif _type == "Adw":
                    from tkadwold.windows.widgets.adw import Adw
                    type = Adw
                elif _type == "AdwTLabel":
                    from tkadwold.windows.theme import AdwTLabel
                    type = AdwTLabel

                if "master" in widgets[index]:
                    master = self.getkey(widgets[index]["master"])
                else:
                    master = None
                self.keys[_name] = type(master=master)
                if "title" in widgets[index]:
                    self.keys[_name].title(widgets[index]["title"])
                if "text" in widgets[index]:
                    self.keys[_name].configure(text=widgets[index]["text"])

    def getkey(self, key):
        return self.keys[key]


if __name__ == '__main__':
    from tkadwold import Adwite

    builder = AdwBuilder(
        {
            "adwite": {
                "theme": "win11",
                "mode": "dark"
            },
            "Widgets": {
                "Root::Adwite": {
                    "title": "AdwBuilder"
                },
                "Label::AdwTLabel": {
                    "master": "Root",
                    "text": "AdwBuilder"
                }
            }
        }
    )
    root = builder.getkey("Root")
    label = builder.getkey("Label")
    label.pack(fill="both")
    root.update()
    root.mainloop()



class RoundRect:
    def __init__(self):
        from tkinter import _default_root, Tk
        self.tk: Tk = _default_root
        self.init1()

    def init1(self):
        """
        初始化圆角矩形模块
        参见：https://wiki.tcl-lang.org/page/Drawing+rounded+rectangles
        """
        self.tk.eval(
            """
proc roundRect2 {w L T Rad width height fill outline tag} {

  $w create oval $L $T [expr $L + $Rad] [expr $T + $Rad] -fill $fill -outline $outline -tag $tag
  $w create oval [expr $width-$Rad] $T $width [expr $T + $Rad] -fill $fill -outline $outline -tag $tag
  $w create oval $L [expr $height-$Rad] [expr $L+$Rad] $height -fill $fill -outline $outline -tag $tag
  $w create oval [expr $width-$Rad] [expr $height-$Rad] [expr $width] $height -fill $fill -outline $outline -tag $tag
  $w create rectangle [expr $L + ($Rad/2.0)] $T [expr $width-($Rad/2.0)] $height -fill $fill -outline $outline -tag $tag
  $w create rectangle $L [expr $T + ($Rad/2.0)] $width [expr $height-($Rad/2.0)] -fill $fill -outline $outline -tag $tag
}
            """
        )

    from tkinter import Canvas

    def draw(self, canvas, x, y, width, height, fill, outline, tag, radius):
        """
        绘制圆角矩形（方式1）

        Args:
            canvas (Canvas): 被绘制的画布
            x (int): 圆角矩形在画布上X轴坐标
            y (int): 圆角矩形在画布上Y轴坐标
            width (int): 圆角矩形的宽度
            height (int): 圆角矩形的高度
            fill (str): 圆角矩形填充颜色
            outline (str): 圆角矩形边框颜色
            tag (str): 圆角矩形标签
            radius (int): 圆角矩形圆角大小
        """
        _rect = self.tk.call("roundRect2", canvas._w, x, y, radius, width, height, fill, outline, tag)
        return _rect

    def draw2(self, canvas, x1, y1, x2, y2, radius, **kwargs):
        """
        绘制圆角矩形（方式2）

        Args:
            canvas (Canvas): 被绘制的画布
            x1 (int): 圆角矩形在画布上X轴开始坐标
            y1 (int): 圆角矩形在画布上Y轴开始坐标
            x2 (int): 圆角矩形在画布上X轴结束坐标
            y2 (int): 圆角矩形在画布上Y轴结束坐标
            radius (int): 圆角矩形圆角大小
        """
        points = [x1 + radius, y1,
                  x1 + radius, y1,
                  x2 - radius, y1,
                  x2 - radius, y1,
                  x2, y1,
                  x2, y1 + radius,
                  x2, y1 + radius,
                  x2, y2 - radius,
                  x2, y2 - radius,
                  x2, y2,
                  x2 - radius, y2,
                  x2 - radius, y2,
                  x1 + radius, y2,
                  x1 + radius, y2,
                  x1, y2,
                  x1, y2 - radius,
                  x1, y2 - radius,
                  x1, y1 + radius,
                  x1, y1 + radius,
                  x1, y1]

        return canvas.create_polygon(points, **kwargs, smooth=True)